/************************************************************************/
/*                                                                      */
/*   Program Name :  ads2k5                                             */
/*                                                                      */
/*     Ads2k5 is a general pupose ADS300+ data file to K5/VSSP          */
/*       format converter                                               */
/*                                                                      */
/*            Ver 0.00  2010-06-03 start based on m5btok5               */
/*            Ver 1.00  2010-06-10 release                              */
/*            Ver 1.01  2010-06-11 Environmetal variable are added      */
/*            Ver 1.02  2010-06-21 Debug in 4x4ch default mode          */
/*            Ver 1.03  2011-02-09 "long int" type were changed to      */
/*                                 "int" type                           */
/*            Ver 1.10  2014-08-12 add 4-bit and 8-bit AD mode          */
/*            Ver 1.11  2014-08-25 improve handling sampling freq       */
/*                                 (set by option should be highest)    */
/*                                                                      */
/*      Copyright (c) 2010-2014  T.Kondo/NICT All Right Reserved        */
/*                                                                      */
/************************************************************************/
/*

     2010-06-03  -- development started

   how to run
          ads2k5 ads_filename [options]
    
     where   ads_filename -- ADS3000 file name
                       
                  ADS3000 file name follows naming rule as follows.

                      others_YYYYDDDHHMMSS.raw
                            where  others -- user defined (any)
                                             this can include a sampling frequency
                                             information (see Note below)
                                   YYYYDDDDHHMMSS -- start time of data
                                   .raw ---- extension for ADS3000 file
                         
              options (any order)

               -c channel   -- pickup channel (1-16) for 1ch conversion mode
                               (same as -1ch option. default 4x4ch mode)
               -u unit      -- pickup unit (1-4) for 4 ch conversion mode
                               (-c option is stronger than this option);
                               (default 4x4ch mode)
               -t YYYYDDDHHMMSS|YYYY/DDD-HH:MM:SS
                  or HHMMSS|HH:MM:SS
                  or YYYYDDD|YYYY/DDD
                       -- set start date and time of data. If this is omitted, 
                          start time is taken from ads_filename.
               -s strat_sec  -- offset time from data head (sec)
                                 (default :  0 )
               -p period     -- data period to convert (sec)
                                 (default : all)
               -adbit adbit   -- # of AD bit for VSSP format data (1,2,4,8) (deflt=1)
               -fsampl fmhz   -- channel sampling frequency (MHz) (deflt=32)

               -1ch ch        -- set K5 data to 1ch mode and pick up 1ch of ADS3000-BBC
                                 (starting at 1)
               -4ch ch1 ch2 ch3 ch4
                              -- set K5 data to 4ch mode and pick up 4 channels from 
                                 ADS3000-BBC
                   When both "-1ch" and "-4ch" option are omitted, ads2k5 converts
                   according to 4x4ch mode, i.e., four K5 files are created.


               -2bit mode     -- select 2bit AD decode table
                                     1: offset binary   0,1,2,3 (default  K5/VSSP)
                                     2: Mark V          0,2,1,3 
                                     3: signed integer  1,0,2,3

               -o k5name     -- K5 file name to be created
                                 (default : see below)
               -d k5dir      -- directory for k5 file out
                                 (default  : ADS3000 directory)
               -vssp         -- force K5 format to VSSP (default is VSSP32)
               -m[onit]        -- moniter run parameters
               -adsbit adsbit      -- ADS AD bits (2|4|8) (deflt=2). if adbit >= 4, adsbit =adbit
               -adsnumch adsnumch  -- ADS # of channels (1|8) (deflt=8)
  
  Naming rule for K5 file created (ADS : original ADS3000 file name. if "-o" option
  is set, ADS is k5name set by "-o" option).  In the ADS name, extention is omitted.
         4x4ch mode
            ADS.k5a ---- for group#1  (CH 01-04)
            ADS.k5b ---- for group#2  (CH 05-08)
            ADS.k5c ---- for group#3  (CH 09-12)
            ADS.k5d ---- for group#4  (CH 13-16)
         4ch mode
            ADS.k5[a|b|c|d]
         1ch mode
            ADS.k5-NN ---- where NN is channel number 01-16

  Enviroment variables
          K5_CH_TABLE  ---   operator defined channel allocation table
          ADSDIR    ---   default path to ADS3000 data
          K5DIR     ---   default path to K5VSSP file out

  Note:
  If ADS3000 file name satisfies following condition, start time and sampling frequency
  are properly estimated from the file name.

          1. File name has a following structure.

               xxx...xxxx_YYYYDDDHHMMSS.raw
                
                    where  xxx...xxxx --- any characters
                           YYYYDDDHHMMSS -- start time (year,day of year,
                                            hour, minute, second)


          2. "xxx...xxxx" includes a sampling information block 
             like "16MHz" or "8Msps" (don't care about
             upper-case or lower-case of characters for MHz or Msps).


          3. This information block should be precede by a non-number ASCII character
             or start of the file name, e.g.,

             ....dbbc_16MHz_.......  or ...xyz.32MSPS.xxxxxx



  Note2:  When compiling on VC++, set stack size at least 4000000(4MB).
         otherwise results in runtime error.


*** sample run parameters ****

D:\chkdata\dbbc.test\dbbc.16ch.2bit.32Msps_2010153105355.raw
D:\data\GRP_data\K+U_DM567606_GRPcut\UsdL_2014206233122_0000_090-140_ch1_8bit_64MspsDeD56.7607_GRPcut.raw -adbit 8 -fsampl 64 -adsnumch 1 -t 2014206233122

*/

#define PROGRAM_VERSION "ads2k5 (Ver 1.11  2014-08-25)"

#include "libk5.h"      // common header for IP-VLBI utilities

#include "libmk5tok5.h"  // library for Mark5 to K5 conversion

/* environment variable for ADS3000 data default directory set */
#define K5_CH_TABLE "K5_CH_TABLE" // the environment variable for operator defined 
                                  // channel allocation table
#define ADSDIR "ADSDIR" // the environment variable for ADS3000 data default directory
#define K5DIR "K5DIR" // the environment variable for K5 file output default directory


/* default parameters */
#define K5NUMCH  4    // default # of channel in K5/VSSP
#define K5ADBIT  1    // default AD bits in K5/VSSP
#define CHSAMPLE 32  // default ch sampling frequency in MHz
#define ADS_BITSTREAM 32 // bit stream for ADS file  (32 bit / sample)
#define DFLT_CH_TABLE {1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16}  // default CH allocation table
#define DFLT_CONVERSION_MODE 3  // default conversion mode 
                                // 1: 1ch mode   2: 4ch mode    3: 4x4ch mode   4:  2x4ch mode
#define DFLT_ADSADBIT 2  // default ADS AD bits         2014-08-12
#define DFLT_ADSNUMCH 8  // default ADS # of channels   2014-08-12


typedef struct ads2k5runp     // run parameters for ADS3000-DBBC to k5 conversion program
{

    int ads_ch; // pick up channel of ADS3000-DBBC data for 1ch mode
    int chtable[16]; 
    int unit;
    int vssp;  // 1: set VSSP format  0: set VSSP32 format

    int soffset;
    int period;
    int fmode;   // conversion out file mode 1: 1ch mode   2: 4ch mode    3: 4x4ch mode   4:  2x4ch mode
    char adsname[256]; 
    char k5name[4][256];
    char k5name_given[256];
    char outdir[256];     
    int k5adbit;
    int k5numch; 
    int adsadbit;    // ADS AD bits       2014-08-12
    int adsnumch;    // ADS # of channels 2014-08-12
    int e2bit; // 2bit data encode mode
    int nbitstream; 
    int itimes[5]; // ADS3000-DBBC data start time
    int tflag;     // 1: time set by option  0: time taken from file name
    double fmhz;   // adopted sampling frequency
    int estim_fmhz; // estimated sampling frequency
    int file_ch_table[4][4]; // channel table for created K5/VSSP file
    int monit;   // ruparameter monitor  1: ON   0: OFF
} ads2k5runp_t;


void vssp_hd_make_ini(vssp_head_t *vpb);
void estim_ads_fsampl_from_fname(char *others, int *fsmhz);
int parse_ads3000_filename(char *fname, int *itime, char *others, int *fsmhz);
int convADS2K5(ads2k5runp_t *runp);
void conv_ads2k5_1ch(int iwADS, int adsch, int *conv2bit, int k5adbit,
                    int *nbit, int *iwK5w);
void conv_ads2k5_4ch(int iwADS, int *chtable, int *conv2bit, int k5adbit,
                    int *nbit, int *iwK5w);
void conv_ads2k5_4x4ch(int iwADS, int *chtable, int *conv2bit, int k5adbit,
                    int *nbit, int *iwK5);
void conv_ads2k5_4x4ch_fast(int iwADS,  int k5adbit, int *nbit, int *iwK5);
void conv_ads2k5_2x4ch_fast(int iwADS,  int k5adbit, int *nbit, int *iwk5);
void conv_ads2k5_1ch_1ch(int iwADS, int nsample, int *conv2bit, int k5adbit, int *nbit, int *iwK5);

void k5name_make_fmode(char *adsname, char *outdir, int fmode, int group, int channel, int adsnumch,  
                 char *k5name);




void time_option_parse(char *cbuf, int *itime)
{
/*
    time option parse 2010-06-09

   [in]
      cbuf --  "YYYYDDDHHMMSS|YYYY/DDD-HH:MM:SS"
                or "HHMMSS|HH:MM:SS"
                or "YYYYDDD|YYYY/DDD"
   [return]
      itime[5] --- year, day of year, hour, minute, second
                   unsatified element is not reset.
    
    
*/
    int len;
    char abuf[10];

    len=strlen(cbuf);

    if(len==17){  //  YYYY/DDD-HH:MM:SS
        strncpy(abuf,cbuf,4);    abuf[4]='\0'; itime[0]=atoi(abuf); // year
        strncpy(abuf,cbuf+5,3);  abuf[3]='\0'; itime[1]=atoi(abuf); // day of year
        strncpy(abuf,cbuf+9,2);  abuf[2]='\0'; itime[2]=atoi(abuf); // hour
        strncpy(abuf,cbuf+12,2); abuf[2]='\0'; itime[3]=atoi(abuf); // minute
        strncpy(abuf,cbuf+15,2); abuf[2]='\0'; itime[4]=atoi(abuf); // second
    } else if(len==13){ // YYYYDDDHHMMSS
        strncpy(abuf,cbuf,4);    abuf[4]='\0'; itime[0]=atoi(abuf); // year
        strncpy(abuf,cbuf+4,3);  abuf[3]='\0'; itime[1]=atoi(abuf); // day of year
        strncpy(abuf,cbuf+7,2);  abuf[2]='\0'; itime[2]=atoi(abuf); // hour
        strncpy(abuf,cbuf+9,2);  abuf[2]='\0'; itime[3]=atoi(abuf); // minute
        strncpy(abuf,cbuf+11,2); abuf[2]='\0'; itime[4]=atoi(abuf); // second
    } else if(len==8){  // HH:MM:SS or YYYY/DDD
        if(cbuf[4]=='/'){ // YYYY/DDD
           strncpy(abuf,cbuf,4);    abuf[4]='\0'; itime[0]=atoi(abuf); // year
           strncpy(abuf,cbuf+5,3);  abuf[3]='\0'; itime[1]=atoi(abuf); // day of year
        } else {   // HH:MM:SS
           strncpy(abuf,cbuf,2);    abuf[2]='\0'; itime[2]=atoi(abuf); // hour
           strncpy(abuf,cbuf+3,2);  abuf[2]='\0'; itime[3]=atoi(abuf); // minute
           strncpy(abuf,cbuf+6,2);  abuf[2]='\0'; itime[4]=atoi(abuf); // second
        }
    } else if(len==6){  // HHMMSS
        strncpy(abuf,cbuf,2);    abuf[2]='\0'; itime[2]=atoi(abuf); // hour
        strncpy(abuf,cbuf+2,2);  abuf[2]='\0'; itime[3]=atoi(abuf); // minute
        strncpy(abuf,cbuf+4,2);  abuf[2]='\0'; itime[4]=atoi(abuf); // second
    } else if(len==7){  // YYYYDDD
        strncpy(abuf,cbuf,4);    abuf[4]='\0'; itime[0]=atoi(abuf); // year
        strncpy(abuf,cbuf+4,3);  abuf[3]='\0'; itime[1]=atoi(abuf); // day of year
    }
}


void param_get_ads2k5(int argc, char *argv[], ads2k5runp_t *runp)                       
{
    int n,i;
    char cbuf[256];
    int ch_table_dflt[]=DFLT_CH_TABLE;
    vparam_t vp;
    //1st param should be ADS3000 file name
    if (argc>=2){
       strcpy(runp->adsname,*(argv+1));
    } else {
       runp->adsname[0]='\0';
    }
     /* check ADS3000 file name is acompanied by directory or not */
    get_true_path(runp->adsname, ADSDIR, "./",cbuf);
    strcpy(runp->adsname,cbuf);

    // initialize parameters

    /* common parameters */
    for(n=0; n<4; n++) runp->k5name[n][0]='\0';
    runp->k5name_given[0]='\0';
    runp->soffset=0;
    runp->period=0;
    runp->vssp=0;

    runp->unit=0;
    runp->outdir[0]='\0';
    runp->fmode=DFLT_CONVERSION_MODE;   // default conversion file mode 1: 1ch  2: 4ch  3: 4x4ch  4: 2x4ch

    runp->k5numch=K5NUMCH;
    runp->k5adbit =K5ADBIT;
    runp->fmhz  =0.0;
    runp->nbitstream=ADS_BITSTREAM;
    for(n=0; n<16; n++) runp->chtable[n]=ch_table_dflt[n];
    runp->ads_ch=0;
    runp->e2bit=1;
    for(n=0; n<5; n++) runp->itimes[n]=0;
    runp->tflag=0;
    runp->monit=0;
    runp->adsadbit=DFLT_ADSADBIT;  // 2014-08-12
    runp->adsnumch=DFLT_ADSNUMCH;  // 2014-08-12

    // environment variable check
    if(getenv(K5_CH_TABLE) != NULL){
        strcpy(cbuf,getenv(K5_CH_TABLE));
        i=line_parse(cbuf, &vp);
        printf("K5_CH_TABLE %d %s\n",i,cbuf);
        if(i > 16 ) i=16;
        if(i > 0){
           for(n=0; n<i; n++) runp->chtable[n]=atoi(vp.param[n]);
        }
    }
    if(getenv(K5DIR) != NULL){
        strcpy(runp->outdir,getenv(K5DIR));
    }


    n=3;
    while(n <=argc) {
        strcpy(cbuf,*(argv+n-1));
        if(strlen(cbuf)==2 && strncmp(cbuf,"-o",2)==0){  // k5 name
            n++;
            if(n > argc) break;
            strcpy(runp->k5name_given,*(argv+n-1));
        }
        if(strncmp(cbuf,"-d",2)==0){  // out directory
            n++;
            if(n > argc) break;
            strcpy(runp->outdir,*(argv+n-1));
            /* env variable is substituted by real one */

            //if((i=substitute_env_path_name(runp->outdir, cbuf))>0){
            //    strcpy(runp->outdir,cbuf);
            //}

        }
        if(strncmp(cbuf,"-t",2)==0){  // time set
            runp->tflag=1;
            n++;
            if(n > argc) break;
            strcpy(cbuf,*(argv+n-1));
            time_option_parse(cbuf, runp->itimes);
        }
        if(strncmp(cbuf,"-u",2)==0){  // unit define
            runp->k5numch=4;
            runp->fmode=2; // 4ch mode
            n++;
            if(n > argc) break;
            runp->unit = atoi(*(argv + n-1 ));
        }
        if(strncmp(cbuf,"-4ch",4)==0){  // K5 is 4ch mode
            runp->k5numch=4;
            runp->fmode=2;
            n++;
            if(n > argc) break;
            runp->chtable[0]=atoi(*(argv + n-1 ));
            n++;
            if(n > argc) break;
            runp->chtable[1]=atoi(*(argv + n-1 ));
            n++;
            if(n > argc) break;
            runp->chtable[2]=atoi(*(argv + n-1 ));
            n++;
            if(n > argc) break;
            runp->chtable[3]=atoi(*(argv + n-1 ));
        }
        if(strncmp(cbuf,"-c",2)==0){  // channel define
            runp->k5numch=1;
            runp->fmode=1; // 1ch mode
            n++;
            if(n > argc) break;
            runp->ads_ch = atoi(*(argv + n-1 ));
            //runp->chtable[0]=atoi(*(argv + n-1 ));
        }
        if(strncmp(cbuf,"-1ch",4)==0){  // K5 is 1ch mode
            runp->k5numch=1;
            runp->fmode=1;
            n++;
            if(n > argc) break;
            runp->ads_ch = atoi(*(argv + n-1 ));
            //runp->chtable[0]=atoi(*(argv + n-1 ));
        }
        if(strlen(cbuf)==2 && strncmp(cbuf,"-s",2)==0){  // start offset
            n++;
            if(n > argc) break;
            runp->soffset = atoi(*(argv + n-1 ));
        }
        if(strncmp(cbuf,"-p",2)==0){  // period
            n++;
            if(n > argc) break;
            runp->period = atoi(*(argv + n-1 ));
        }
        if(strncmp(cbuf,"-fs",3)==0){  // ADS sampling frequency
            n++;
            if(n > argc) break;
            runp->fmhz=atof(*(argv + n-1 ));
        }
        if(strncmp(cbuf,"-adb",4)==0){  // K5 AD bits
            n++;
            if(n > argc) break;
            runp->k5adbit=atoi(*(argv + n-1 ));
            if(runp->k5adbit==4) runp->fmode=4;  // 2x4ch mode  2014-08-12
            if(runp->k5adbit==8) runp->fmode=2;  // 1file 4ch mode  2014-08-12
        }
        if(strncmp(cbuf,"-bs",3)==0){  // ADS bit stream   (always 32)
            n++;
            if(n > argc) break;
            runp->nbitstream=atoi(*(argv + n-1 ));
        }
        if(strncmp(cbuf,"-2bit",5)==0){  // 2bit AD decode mode set
            n++;
            if(n > argc) break;
            runp->e2bit = atoi(*(argv + n-1 ));
        }
        if(strlen(cbuf)==5 && strncmp(cbuf,"-vssp",5)==0){  // set VSSP format
            runp->vssp=1;
        }
        if(strncmp(cbuf,"-m",2)==0){  // set monitor ON
            runp->monit=1;
        }
        if(strncmp(cbuf,"-adsb",5)==0){  // ADS AD bits   2014-08-12
            n++;
            if(n > argc) break;
            runp->adsadbit=atoi(*(argv + n-1 ));
        }
        if(strncmp(cbuf,"-adsn",5)==0){  // ADS # of channels   2014-08-12
            n++;
            if(n > argc) break;
            runp->adsnumch=atoi(*(argv + n-1 ));
        }
        n++;
    }
    if(runp->k5adbit >=4) runp->adsadbit=runp->k5adbit;  // 2014-08-12
    if(runp->adsnumch ==1) {  // 2014-08-12
        runp->ads_ch=1;
        runp->fmode=1;
    }
}

void print_usage(void)
{
    printf("%s\n",PROGRAM_VERSION);
    printf("\n");
    printf("    ads2k5 ads_filename [options]\n");
    printf("\n");
    printf("where   ads_filename -- ADS3000-DBBC data file name\n");
    printf("      options (any order)\n");
    printf("        -c channel    -- pickup channel (1-16) for 1ch conversion mode\n");
    printf("                          (same as -1ch option. default 4x4ch mode)\n");
    printf("        -u unit       -- pickup unit (1-4) for 4 ch conversion mode\n");
    printf("                          (-c option is stronger than this option)\n");
    printf("                          (default 4x4ch mode)\n");
    printf("        -o k5name     -- K5 file name to be created\n");
    printf("                          (default : see below)\n");
    printf("        -d k5dir      -- directory for k5 file out\n");
    printf("                          (default  : mark5 directory)\n");
//    printf("        -bs nbitsream -- set ADS3000 bit-stream (always 32)\n");
    printf("        -fsampl fmhz  -- channel sampling frequency (MHz) (deflt=%d)\n",CHSAMPLE);
    printf("        -adbit adbit  -- # of AD conversion bit (deflt=%d)\n",K5ADBIT);
    printf("        -1ch ch       -- set K5 data to 1ch mode and \n");
    printf("                          set pick up ADS3000-DBBC ch (starting at 1)\n");
    printf("        -4ch ch1 ch2 ch3 ch4\n");
    printf("                      -- set K5 data to 4 ch mode and  \n");
    printf("                           set pick up channels from ADS3000-DBBC\n");
    printf("                         default: convert to 4x4ch K5 files\n"); 
    printf("        -2bit mode    -- select 2bit AD decode table\n");
    printf("                           1: offset binary   0,1,2,3 (no conversion: deflt)\n");
    printf("                           2: Mark V standard 0,2,1,3\n");
    printf("                           3: signed integer  1,0,2,3\n");
    printf("        -t YYYYDDDHHMMSS|YYYY/DDD-HH:MM:SS\n");
    printf("            or HHMMSS|HH:MM:SS\n");
    printf("            or YYYYDDD|YYYY/DDD\n");
    printf("               -- set start date and time of data. If this is omitted, \n");
    printf("                  start time is taken from ads_filename.\n");
    printf("        -s strat_sec  -- offset time from data head (sec)\n");
    printf("                          (default :  0 )\n");
    printf("        -p period     -- data period to convert (sec)\n");
    printf("                          (default : all)\n");
    printf("        -vssp         -- force K5 format to VSSP (default is VSSP32)\n");
    printf("        -monit        -- monitor run parameters\n");
    printf("        -adsbit adsbit      -- ADS AD bits (2|4|8) (deflt=2). \n");
    printf("                               if adbit >= 4, adsbit =adbit\n");
    printf("        -adsnumch adsnumch  -- ADS # of channels (1|8) (deflt=8)\n");
   printf("   Naming rule for K5 file created (ADS : ADS3000 file name\n");
    printf("            or name given by -o option\n");
    printf("      4x4ch mode\n");
    printf("        ADS.k5a ---- for group#1 (ch01-04)\n");
    printf("        ADS.k5b ---- for group#2 (ch05-08)\n");
    printf("        ADS.k5c ---- for group#3 (ch09-12)\n");
    printf("        ADS.k5d ---- for group#4 (ch13-16)\n");
    printf("      4ch mode\n");
    printf("        ADS.k5[a|b|c|d]\n");
    printf("      1ch mode\n");
    printf("        ADS.k5-NN ---- where NN is channel number 01-16\n");
    printf("   Enviroment variables\n");
    printf("        %s    ---   operator defined CH allocation table ( %s )\n",
        K5_CH_TABLE,getenv(K5_CH_TABLE));
    printf("        %s    ---   default path to ADS3000 data ( %s )\n",
        ADSDIR,getenv(ADSDIR));
    printf("        %s    ---   default path to K5VSSP file out ( %s )\n",
        K5DIR,getenv(K5DIR));
    printf("\n");
    printf("  Note:\n");
    printf("  If ADS3000 file name satisfies following condition, start time and \n");
    printf("  sampling frequency are properly estimated from the file name.\n");
    printf("\n");
    printf("          1. File name has a following structure.\n");
    printf("\n");
    printf("               xxx...xxxx_YYYYDDDHHMMSS.raw\n");
    printf("                \n");
    printf("                    where  xxx...xxxx --- any characters\n");
    printf("                           YYYYDDDHHMMSS -- start time (year,day of year,\n");
    printf("                                            hour, minute, second)\n");
    printf("\n");
    printf("          2. \"xxx...xxxx\" includes a sampling information block \n");
    printf("             like \"16MHz\" or \"8Msps\" (don't care about\n");
    printf("             upper-case or lower-case of characters for MHz or Msps).\n");
    printf("\n");
    printf("          3. This information block should be precede by a non-number \n");
    printf("             ASCII character or start of the file name, e.g.,\n");
    printf("\n");
    printf("             ....dbbc_16MHz_.......  or ...xyz.32MSPS.xxxxxx\n");
    printf("\n");
}

void monit_runp(ads2k5runp_t *runp)
// run parameter monitor
{

       printf(" ---------- RUN CONDITION ---------\n");
       printf("   start time: %4d/%03d %02d:%02d:%02d\n",runp->itimes[0],
           runp->itimes[1],runp->itimes[2],runp->itimes[3],runp->itimes[4]);
       printf("   adsname   : %s\n",runp->adsname);
       printf("   k5name    : %s\n",runp->k5name_given);
       printf("   outdir    : %s\n",runp->outdir);
       printf("   bitstream : %d\n",runp->nbitstream);
       printf("   fmhz      : %f\n",runp->fmhz);
       printf("   k5adbit   : %d\n",runp->k5adbit);
       printf("   k5numch   : %d\n",runp->k5numch);
       printf("   adschannels: %d %d %d %d\n",runp->chtable[0],
           runp->chtable[1],runp->chtable[2],runp->chtable[3]);
       printf("              : %d %d %d %d\n",runp->chtable[4],
           runp->chtable[5],runp->chtable[6],runp->chtable[7]);
       printf("              : %d %d %d %d\n",runp->chtable[8],
           runp->chtable[9],runp->chtable[10],runp->chtable[11]);
       printf("              : %d %d %d %d\n",runp->chtable[12],
           runp->chtable[13],runp->chtable[14],runp->chtable[15]);
       printf("   2bit decode mode : %d\n",runp->e2bit);
       printf("   ads_ch    : %d\n",runp->ads_ch);
       printf("   unit#     : %d\n",runp->unit);
       printf("   soffset   : %d\n",runp->soffset);
       printf("   period    : %d\n",runp->period);
       printf("   vssp      : %d\n",runp->vssp);
       printf(" -------------------------------------\n");
    printf("\n");
}


int main(int argc, char *argv[])
{
       int i,n;
       //int eflag;
       double telapsed;           // to measure processing time
       char fname[256];
       //char fpath[256];
       char others[256];
       char program_version[60];
       int itime[5];
       time_t tstart, tfinish;    
       ads2k5runp_t runp;

       if(argc == 1 || (argc == 2 && strncmp(*(argv+1),"--help",6)==0 )){ 
          print_usage();
          exit(0);
       }
       if(argc == 2 && strncmp(*(argv+1),"--ver",5)==0 ){ 
          printf("\n");
          printf("%s\n",PROGRAM_VERSION);
          printf("\n");
          exit(0);
       } 
       if (argc>=2){
          if(strncmp(*(argv+1),"env",3)==0 || strncmp(*(argv+1),"--env",5)==0){
          printf("   Enviroment variables\n");
          printf("        %s ---   operator defined channel allocation table\n",K5_CH_TABLE);
          printf("                      ( %s )\n",getenv(K5_CH_TABLE));
          printf("        %s ---   default directory for ADS3000 data\n",ADSDIR);
          printf("                      ( %s )\n",getenv(ADSDIR));
          printf("        %s ---   default directory for K5VSSP file out\n",K5DIR);
          printf("                      ( %s )\n",getenv(K5DIR));
          printf("\n");
          exit(0);
          }
       }

       strcpy(program_version,PROGRAM_VERSION);

       /* run parameters get */
       param_get_ads2k5(argc, argv, &runp); 

       i=parse_ads3000_filename(runp.adsname, itime, others, &runp.estim_fmhz);
       if(runp.tflag==0) memcpy(runp.itimes,itime,5*sizeof(int));

       printf("%s \n",runp.adsname);
       printf(" %4d/%03d %02d:%02d:%02d\n",runp.itimes[0],runp.itimes[1],
           runp.itimes[2],runp.itimes[3],runp.itimes[4]);
       printf(" Others : %s\n",others);
       printf(" Estimated sampling frequency from file name (MHz) : %d\n",runp.estim_fmhz);

       // sampling frequency decision  (given by operator has the highest priority)

       if(runp.estim_fmhz == 0) {  // in case of that file name doesn't include sampling freq
           if(runp.fmhz == 0.0){
               runp.fmhz=(double)CHSAMPLE;  // use of program default
           }
           //printf(" Sampling frequency adopted (MHz) : %d\n",(int)runp.fmhz);
       //} else {
       } else if(runp.fmhz == 0){              // 2014-08-25
           runp.fmhz=(double)runp.estim_fmhz;  // use of estimated sampling frequency
       }
       printf(" Sampling frequency adopted (MHz) : %d\n",(int)runp.fmhz);

       if(runp.monit==1) monit_runp(&runp);


       /* out directory name create if not set */

       if(runp.outdir[0]=='\0'){
          parse_dir_and_name(runp.adsname, runp.outdir, fname); // debug
          //printf("A %s\n",runp.adsname);
          //printf("A %s\n",runp.outdir);
          //printf("A %s\n",fname);
       }

       /**************************************/
       /* Entry of Conversion mode operation */ 
       /**************************************/



       /* Program Name Display */
       printf("************************************************************\n");
       printf("*    ADS3000-DBBC to K5/VSSP Data Format Converter         *\n");
       printf("*      %s  by  T.KONDO/NICT     *\n",program_version);
       printf("************************************************************\n");
       printf("\n");


       tstart= time(NULL);


       // set run condition to make clear
       if(runp.fmode==1){   // 1ch mode
           runp.k5numch=1;
           runp.unit=0;
       } else {  // 4ch mode or 4x4ch mode
           runp.k5numch=4;
       }


   /********************************************/
   /* Conversion Start                         */ 
   /********************************************/

       i=convADS2K5(&runp);

       if(i < 0) goto exitend;

       printf("  K5 file(s) created is as follows.\n");
       printf("  =========================================================================\n");
       printf("   Pick Up CHs         File Name\n");
       printf("  -------------------------------------------------------------------------\n");
       for(n=0; n<4; n++){
           if(runp.k5name[n][0]!='\0'){
              printf("  ");
              for(i=0; i<4; i++){
                 if(runp.file_ch_table[n][i] > 0){
                     printf(" %2d",runp.file_ch_table[n][i]);
                 } else {
                     printf("   ");
                 }
              }
              printf("  %s\n",runp.k5name[n]);
           }
       }
       printf("  =========================================================================\n");
       printf("\n");

       tfinish= time(NULL);
       telapsed = difftime(tfinish,tstart);
       printf("Time elapsed (sec) for One obs process is %f\n",telapsed);

exitend: 
       return 0;
}


void estim_ads_fsampl_from_fname(char *others, int *fsmhz)
{
/*
      estimate sampling frequency of ADS3000-DBBC
      from file name
         key word  "MHz"  "Msps"

         nnMHz   or   nn Msps

     [return]
         fsmhz  --  8, 16, 32, 64  for success in estimation
                    0 for fault in estimation

*/
    int i,n,k1,k2,fs,len,len1,iok;
    char cbuf[256],abuf[10];
    char *key_table[2];
    char *key1="MHZ", *key2="MSPS";

    key_table[0]=key1; key_table[1]=key2;
    fs=0;

    strcpy(cbuf, others);

    // convert all charcters to upper case letter

    conv2large(cbuf);

    // found key word

    iok=0;
    k1=0;

    len=strlen(cbuf);



    for(i=0; i<2; i++){
       len1=strlen(key_table[i]);
       if(len <= len1) goto exit2return;
       for(n=0; n<=len-len1; n++){
          if(strncmp(cbuf+n,key_table[i],len1)==0){ // key word found
              k1=n;
              iok=1;
              break;
          }
       }
       if(iok==1) break;
    }
    if(iok==0) goto exit2return;
    if(k1==0) goto exit2return;

    // get sampling frequency
    // at first found non number charcater
    k2=0;
    for(n=k1-1; n>=0; n--){
        if(cbuf[n] < 48 || cbuf[n] >57){ //  another than '0'-'9'
            if(cbuf[n] != '+' && cbuf[n] != '-'){ // non number character
                k2=n;
                break;
            }
        }
    }
    if(k2 == k1-1) goto exit2return;

    strncpy(abuf,cbuf+k2+1,k1-k2-1); 
    abuf[k1-k2-1]='\0';

    fs=atoi(abuf);

exit2return:;

    *fsmhz=fs;

}



int parse_ads3000_filename(char *fname, int *itime, char *others, int *fsmhz) 
/******************************************************************************************/
/*     T.Kondo       2010-06-07                                                           */
/*                                                                                        */
/*   Parse ADS3000 file name to time data and others                                      */
/*                                                                                        */
/*    IN                                                                                  */
/*        fname --  ADS3000 file name                                                     */
/*    RETURN                                                                              */
/*        0: OK,  -1: no time data in the file name or no "."                             */
/*        itime[5] --  Time (4digit year, total day, hour, minute, second)                */
/*        others  --  other portion of file name                                          */
/*                                                                                        */
/*    Note:  Definition of ADS3000 file name is as follows                                */
/*                                                                                        */
/*            others_YYYYDDDHHMMSS.raw                                                    */
/*                                                                                        */
/******************************************************************************************/
{
    int len,i,n1,n2,iret;
    char abuf[10], ctime[20]; 
    len=strlen(fname);

    iret=0;
    for(i=0; i<5; i++) itime[i]=0;
    others[0]='\0';

    /* get YYYYDDDHHMMSS portion */

    // find last "." in the name
    n2=-1;
    for(i=len-1; i>=0; i--){
       if(fname[i]=='.'){
           n2=i;
           break;
       }
    }
    if(n2 < 0){    // no "." in the file name
        iret=-1;
        goto exit2end;
    }
    n1=n2-13;
    if(n1 < 0){  // no time data
        iret=-1;
        goto exit2end;
    }

    // get YYYYDDDHHMMSS

    strncpy(ctime,fname+n1,13);
    ctime[13]='\0';

    // parse to time data
    strncpy(abuf,ctime,4);    abuf[4]='\0'; itime[0]=atoi(abuf);   // YYYY 
    strncpy(abuf,ctime+4,3);  abuf[3]='\0'; itime[1]=atoi(abuf);   // DDD
    strncpy(abuf,ctime+7,2);  abuf[2]='\0'; itime[2]=atoi(abuf);   // HH
    strncpy(abuf,ctime+9,2);  abuf[2]='\0'; itime[3]=atoi(abuf);   // MM
    strncpy(abuf,ctime+11,2); abuf[2]='\0'; itime[4]=atoi(abuf);   // SS

    // get others
    strncpy(others,fname,n1);
    others[n1]='\0';

    // check last character wheter "_" or not
    if(others[n1-1]=='_') others[n1-1]='\0';

exit2end:;

    estim_ads_fsampl_from_fname(others, fsmhz);
    return iret;
    //printf("debug %d %s\n",k,dir);
}

int convADS2K5(ads2k5runp_t *runp)
/*
    ADS3000-DBBC to K5 conversion  2014-08-12
*/
{
    int pcount,icount,opos,nbit;

    unsigned int idata[2500];  // for data field 2500 X 32 bits
    int nword=2500;  //   # of 32bit data words in a Mark5B frame 
    unsigned int iw;
    double fsample;  // sampling frequency
    int nsample;     // # of smaples per sec
    int hmax;

    int iwk5[4],iwk5w;
    int i,n,nf;
    int iread,iwrite;
    int itime[6];
    //int numch;
    //int ads_adbit=2;   // AD bits in a ADS3000-DBBC (always 2)
    int ads_adbit;   // AD bits in a ADS3000-DBBC (2 or 4)
    int k5adbit;       // AD bits in a K5 format (1 or 2)
    int k5numch;     // # of ch in a K5 format data (1 or 4)
    int ads_chmax;     // total ADS3000-DBBC ch # in a ADS3000-DBBC format (16)
    int ads_ch;        // work ADS ch # in a ADS3000-DBBC format ( <= 16)
                     //  or pick-up ADS3000-DBBC ch in case of K5 1ch mode
    int nthsample; // 2014-08-12
    int nfile,unit;
    int ierr;
    int *obuf[4];
    int nwrite=1024;  // obuf size to write
    int nwordsec;  // number of 32-bit words in a sec
    int bytesec;   // number of bytes in a second data
    vssp_head_t vpb;
    FILE *fo[4];
    FILE *fin;

    int *conv2bit;
    int conv2bit_1[]={0,1,2,3};   // offset binary (no conversion 2 bit: default)
    int conv2bit_2[]={0,2,1,3};   //  conversion 2 bit (according to Mark V)
    int conv2bit_3[]={1,0,2,3};   //  conversion 2 bit singed integer

    int chtabl[16];  // ch pick up table  contents must be prepare from out side

    ierr=0;
    ads_adbit=runp->adsadbit;                            // 2014-08-12
    //if(runp->k5adbit==4) ads_adbit=4;       // 2014-08-12

    if(runp->e2bit==2){
       conv2bit=conv2bit_2;
    } else if(runp->e2bit==3){
       conv2bit=conv2bit_3;
    } else {
       conv2bit=conv2bit_1;
    }

    memcpy(chtabl,runp->chtable,16*sizeof(int));

    nfile=1;
    if(runp->fmode==3) nfile=4;
    if(runp->fmode==4) nfile=2;     // 2014-08-12

    fsample=runp->fmhz*1.0e6;
    nsample=(int)fsample;
    nwordsec=nsample;
    bytesec=4*nwordsec;
    k5adbit=runp->k5adbit;   // convert AD bit (1 or 2 or 4)
    //numch=runp->numch;
    k5numch=runp->k5numch;

    ads_ch=runp->ads_ch;     // pick up ADS channel in case of 1ch mode

    //ads_chmax=runp->nbitstream/runp->adbit;
    ads_chmax=16;
    if(ads_adbit==4) ads_chmax=8;         // 2014-08-12
    if(ads_adbit==8) ads_chmax=4;         // 2014-08-12
    if(runp->adsnumch==1) ads_chmax=1;    // 2014-08-12

    for(n=0; n<4; n++){
        for(i=0; i<4; i++){
            runp->file_ch_table[n][i]=0;
        }
    }

    /* allocate memories */
    for(n=0; n<nfile; n++){
       obuf[n]=(int *)malloc(nwrite*sizeof(int));
    }
    //mk5.header=(int *)malloc(5*mk5.tot_track*sizeof(int));
    //mk5.head180=(int *)malloc(mk5.nbithead*mk5.tot_track*sizeof(int));

    /* first time set */

    memcpy(itime, runp->itimes,5*sizeof(int));
    itime[5]=0;  // set msec 0  and always 0
    vpb.year=itime[0]; vpb.totalday=itime[1];

    /* ADS3000-DBBC data file open */
    if( (fin  = fopen( runp->adsname, "rb" )) == NULL ){
           printf("  ADS3000-DBBC File %s   Open Error! \n",runp->adsname);
           ierr=-1;
           goto exit2return;
    }
    printf("  ********************** ADS3000-DBBC Data File Information ******************\n");
    printf("   File : %s\n",runp->adsname);
    printf("   Start time    : %d/%03d %02d:%02d:%02d\n",
               itime[0],itime[1],itime[2],itime[3],itime[4]);


    /* forward N sec if set so */
    if(runp->soffset > 0) {
        printf("      given offset (sec) =   %d\n",runp->soffset);
        printf("       so forwarded by %d sec \n",runp->soffset);
        for(n=0; n<runp->soffset; n++){
            fseek(fin, bytesec,SEEK_CUR);
            if( ferror( fin ) ){
               printf("      error occurred during forward\n");
               ierr= -1;
               i=fclose( fin) ;
               goto exit2return;
            }
        }
        itime[4]+=runp->soffset;         // second
        while(itime[4]>=60){
           itime[4]-=60;
           itime[3]++;     // minutes
           if(itime[3]==60){
               itime[3]=0;
               itime[2]++;  // hour
               if(itime[2]==24){
                   itime[2]=0;
                   itime[1]++;  // day of year
               }
           }
        }
        printf("   New start time: %d/%03d %02d:%02d:%02d\n",
               itime[0],itime[1],itime[2],itime[3],itime[4]);
    }
    printf("   Estimated sampling frequency from file name (MHz) : %d\n",runp->estim_fmhz);
    printf("   Adopted sampling frequency (MHz)                  : %d\n",(int)runp->fmhz);
    printf("  ****************************************************************************\n");

    printf("\n");
    printf("  ****************** Conversion Condition ************************************\n");
    if(runp->vssp==1){
        printf("    Format:  K5/VSSP\n");
    } else {
        printf("    Format:  K5/VSSP32\n");
    }
    if(runp->fmode==1){
        printf("    Mode  :  1ch mode     Pick-up CH# : %d\n",runp->ads_ch);
        runp->file_ch_table[0][0]=runp->ads_ch;
    } else if(runp->fmode==2){
        printf("    Mode  :  4ch mode     ");
        if(runp->unit>0){
            printf("Pick-up UNIT# : %d\n",runp->unit);
            for(i=0; i<4; i++) runp->file_ch_table[0][i]=runp->chtable[(runp->unit-1)*4+i];
        } else {
            printf("Pick-up CHs  : %d %d %d %d\n",
                runp->chtable[0],runp->chtable[1],runp->chtable[2],runp->chtable[3]);
            for(i=0; i<4; i++) runp->file_ch_table[0][i]=runp->chtable[i];
        }
    } else if(runp->fmode==4){                           // 2014-08-12
        printf("    Mode  :  2x4ch mode     \n");
        for(n=0; n<4; n++){
            for(i=0; i<4; i++) {
                runp->file_ch_table[n][i]=runp->chtable[n*4+i];
            }
        }
    } else {
        printf("    Mode  :  4x4ch mode     \n");
        for(n=0; n<4; n++){
            for(i=0; i<4; i++) {
                runp->file_ch_table[n][i]=runp->chtable[n*4+i];
            }
        }
    }
    printf("    Fsample (MHz) = %f    AD-bits after conversion = %d\n",fsample/1.0e6,k5adbit);
    printf("    Conversion period (sec) :  ");
    if(runp->period > 0) {
        printf("%d\n",runp->period);
    } else {
        printf("ALL\n");
    }
    printf("\n");


    /* output file create */


    unit=runp->unit;
    printf("    Following K5 Data File will be created :\n");
    printf("    =========================================================================\n");
    printf("     Pick Up CHs         File Name\n");
    printf("    -------------------------------------------------------------------------\n");
    for(n=0; n<nfile; n++){
        //if(runp->fmode==3) unit=n+1;
        if(runp->fmode==3 || runp->fmode==4) unit=n+1;   // 2014-08-12
        if(runp->k5name_given[0]=='\0'){
           k5name_make_fmode(runp->adsname, runp->outdir, runp->fmode, unit, 
               runp->ads_ch, runp->adsnumch, runp->k5name[n]);
        } else {
           k5name_make_fmode(runp->k5name_given, runp->outdir, runp->fmode, unit, 
               runp->ads_ch, runp->adsnumch,runp->k5name[n]);
        }
        printf("    ");
        for(i=0; i<4; i++){
            if(runp->file_ch_table[n][i] > 0){
                printf(" %2d",runp->file_ch_table[n][i]);
            } else {
                printf("   ");
            }
        }
        printf("  %s\n",runp->k5name[n]);
        if( (fo[n]  = fopen( runp->k5name[n], "wb" )) == NULL ){
           printf("      File  %s  Open Error! \n",runp->k5name[n]);
           ierr= -2;
           goto exit2return;
        }
    }
    printf("    -------------------------------------------------------------------------\n");
    printf("  ****************************************************************************\n");

    pcount=0; // converted data period counter (in sec)

    /* K5 header data creation */

    vpb.format=2;   // 1: VSSP   2: VSSP32 format
    if(runp->vssp==1) vpb.format=1;

    hmax=8;   // # of header bytes
    if(vpb.format==2) hmax=32;

    vpb.adbit=k5adbit;
    vpb.numch=k5numch;
    vpb.sfreq=(int)(fsample/1000);

    vpb.v_major=0;
    vpb.v_minor=0;
    vpb.aux_fmt=2;
    vpb.aux_size=20; 
    vpb.filter=0;

    /* set host name "ADS2K5" instead of real Host name */
    strcpy(vpb.aux_hostname,"ADS2K5  ");
    strcpy(vpb.aux_station,"ADS-DBBC");
    strcpy(vpb.aux_sid2,"  ");

    printf("  Conversion Started!\n\n");

    /* data monitor */
    printf("\n");
    printf("  ****************************************************************\n");
    printf("                     ADS3000-DBBC Data\n");
    //printf("                           TIME CODE  CRC-  \n");
    //printf("   HH:MM:SS   SYNC   UUUUTFFF MJDsssss.ssss  16  DATA#1   DATA#2\n");
    printf("   HH:MM:SS  DATA#1   DATA#2   DATA#3   DATA#4   DATA#5   DATA#6\n");
    printf("  ----------------------------------------------------------------\n");
       

    nbit=0;
    icount=0;  // 32bit word counter
    opos=0;    // out buffer writing position

    while(1){


        /* ADS3000-DBBC supposed 1 frame read */

        iread = fread( idata, sizeof( unsigned int ), nword, fin );
        if (iread ==0 ) { // EOF
           goto exithere; 
        }

        if(icount==0){   // K5/VSSP header preparation 

            /* info display */
            printf("   %02d:%02d:%02d ",itime[2],itime[3],itime[4]);
            for (i=0; i<6; i++){           
                 printf("%08X ",idata[i]);
            } 
            printf("\n");

            /* K5 header generate */

            vpb.seconds=itime[2]*3600+itime[3]*60+itime[4];
            vpb.totalday=itime[1];
            vssp_hd_make_ini(&vpb);
            i=0;
            while(i < hmax){
                iw=conv4byte2int(vpb.bhead+i);
                for(nf=0; nf<nfile; nf++) obuf[nf][opos]=iw;
                opos++;
                if(opos == nwrite){
                   for(nf=0; nf<nfile; nf++) iwrite=fwrite(obuf[nf],sizeof(int),nwrite,fo[nf]);
                   opos=0;
                }
                i+=4;
            }
        }


        if(runp->adsnumch==1){  // 2014-08-12
           icount+=(iread*32/runp->adsadbit);
        } else {
           icount+=iread;
        }

        if(icount == nsample){ // then time update
            itime[4]++;         // second
            if(itime[4]==60){
                itime[4]=0;
                itime[3]++;     // minutes
                if(itime[3]==60){
                    itime[3]=0;
                    itime[2]++;  // hour
                    if(itime[2]==24){
                        itime[2]=0;
                        itime[1]++;  // day of year
                    }
                }
            }
            icount=0;
            pcount++;
        }

        /* convert to K5 data */

        for(n=0; n<nword; n++){   // nword = 2500 
            iw=idata[n];

            if(nfile == 1){
                iwk5w=obuf[0][opos];

                if(k5numch ==1){

                    /* K5 1ch mode */
                    if(runp->adsnumch==1){ // 2014-08-12

                        for(nthsample=1; nthsample <= 32/k5adbit; nthsample++){
                            conv_ads2k5_1ch_1ch(iw, nthsample, conv2bit, k5adbit, &nbit, &iwk5w);
                            obuf[0][opos]=iwk5w;
                            if(nbit==32){
                                nbit=0;
                                opos++;
                            }
                        }
                    } else {
                         conv_ads2k5_1ch(iw, ads_ch, conv2bit, k5adbit, &nbit, &iwk5w);
                         obuf[0][opos]=iwk5w;
                         if(nbit==32){
                            nbit=0;
                            opos++;
                         }
                    }
                } else {
                   /* convert to K5 4ch mode */
                     conv_ads2k5_4ch(iw, chtabl, conv2bit, k5adbit,  &nbit, &iwk5w);
                     obuf[0][opos]=iwk5w;
                     if(nbit==32){
                        nbit=0;
                        opos++;
                     }
                }

                if(opos == nwrite){
                   iwrite=fwrite(obuf[0],sizeof(int),nwrite,fo[0]);
                   opos=0;
                }
            } else {    // 4x4ch mode

                for(nf=0; nf<nfile; nf++) iwk5[nf]=obuf[nf][opos];


                if(runp->fmode==4){  // 2014-08-12
                    conv_ads2k5_2x4ch_fast(iw, k5adbit, &nbit, iwk5);
                } else {
                    conv_ads2k5_4x4ch_fast(iw, k5adbit, &nbit, iwk5);
                }

                for(nf=0; nf<nfile; nf++) obuf[nf][opos]=iwk5[nf];
                if(nbit==32){
                     nbit=0;
                     opos++;
                }

                if(opos == nwrite){
                    for(nf=0; nf<nfile; nf++) iwrite=fwrite(obuf[nf],sizeof(int),nwrite,fo[nf]);
                    opos=0;
                }
            }
        }
        if(runp->period > 0 && pcount >= runp->period) goto exithere;
    }
exithere:
    i=fclose( fin) ;
    // rest of data write
    if(opos != 0){
       for(nf=0; nf<nfile; nf++) iwrite=fwrite(obuf[nf],sizeof(int),opos,fo[nf]);
    }
    for(nf=0; nf<nfile; nf++)i=fclose( fo[nf] ) ;
    printf("  ****************************************************************\n");
    printf("\n");
exit2return:
       //printf("  K5 file (%s) has been created.\n\n",runp->k5name);
    for(nf=0; nf<nfile; nf++) free(obuf[nf]);

    return ierr;
}


void conv_ads2k5_1ch(int iwADS, int adsch, int *conv2bit, int k5adbit,
                    int *nbit, int *iwK5)
/*
    ADS3000-DBBC to K5 conversion 1ch mode engine 2010-06-07 

        ADS3000-DBBC format is fixed to as follows

             nbitstream : 32
             ads_adbit  :  2
             chmax      : 16

  [in]
     iwADS       --- ADS3000-DBBC   a 32bit word data
     adsch       --- ADS3000-DBBC   ch# to be picked up (starting at 1)
                        in case of non-existing ch#, zero data is
                        filled in K5 data
     conv2bit    --- 2bit data conversion table
     k5adbit     --- K5       AD bits  (1 or 2)
     nbit        --- already filled out bits in iwK5 (starting at 0 and <32) 
     iwK5        --- K5 a 32bit word data

  [return]
     nbit        --- already filled out bits in iwK5 (maximum is 32) 
     iwK5        --- K5 a 32bit word data

*/
{
    int m;
    int iw, ik5, nb, ipick;

    int imask[]={0xFFFFFFFE,0xFFFFFFFD,0xFFFFFFFB,0xFFFFFFF7,
                 0xFFFFFFEF,0xFFFFFFDF,0xFFFFFFBF,0xFFFFFF7F,
                 0xFFFFFEFF,0xFFFFFDFF,0xFFFFFBFF,0xFFFFF7FF,
                 0xFFFFEFFF,0xFFFFDFFF,0xFFFFBFFF,0xFFFF7FFF,
                 0xFFFEFFFF,0xFFFDFFFF,0xFFFBFFFF,0xFFF7FFFF,
                 0xFFEFFFFF,0xFFDFFFFF,0xFFBFFFFF,0xFF7FFFFF,
                 0xFEFFFFFF,0xFDFFFFFF,0xFBFFFFFF,0xF7FFFFFF,
                 0xEFFFFFFF,0xDFFFFFFF,0xBFFFFFFF,0x7FFFFFFF};
    int ibset[]={0x00000001,0x00000002,0x00000004,0x00000008,
                 0x00000010,0x00000020,0x00000040,0x00000080,
                 0x00000100,0x00000200,0x00000400,0x00000800,
                 0x00001000,0x00002000,0x00004000,0x00008000,
                 0x00010000,0x00020000,0x00040000,0x00080000,
                 0x00100000,0x00200000,0x00400000,0x00800000,
                 0x01000000,0x02000000,0x04000000,0x08000000,
                 0x10000000,0x20000000,0x40000000,0x80000000};

    int imask2[]={0xFFFFFFFC,0xFFFFFFF3,0xFFFFFFCF,0xFFFFFF3F,
                  0xFFFFFCFF,0xFFFFF3FF,0xFFFFCFFF,0xFFFF3FFF,
                  0xFFFCFFFF,0xFFF3FFFF,0xFFCFFFFF,0xFF3FFFFF,
                  0xFCFFFFFF,0xF3FFFFFF,0xCFFFFFFF,0x3FFFFFFF};

    int chmax=16;
    //chmax=nbitstream/adbit;

    iw=iwADS;
    ik5= *iwK5;
    nb=*nbit;


    if(adsch > chmax){
       ipick=0;
    } else {
       m=(adsch-1)*2;
       ipick= (int)(iw >> m) & 0x3;
    }
    
    if(k5adbit==2){  // convert to K5 2bit mode
       ipick=conv2bit[ipick];
       ik5 = (ik5 & imask2[nb/2]) | (ipick << nb);
       nb+=2;
    } else {    // pick up 1bit data
       ipick >>= 1;   // MSB get
       ik5 = (ik5 & imask[nb]) | (ipick << nb);
       nb++;
    }
    *nbit=nb;
    *iwK5=ik5;
}


void conv_ads2k5_1ch_1ch(int iwADS, int nthsample, int *conv2bit, int k5adbit,
                    int *nbit, int *iwK5)
/*
    ADS3000-DBBC (1ch mode) to K5 conversion 1ch mode engine 2014-08-12 

        ADS3000-DBBC format is fixed to as follows

             nbitstream : 32
             ads_adbit  :  2|4|8
             chmax      :  1

  [in]
     iwADS       --- ADS3000-DBBC   a 32bit word data
     nthsample     --- position of sample data in 32-bit word (starting from 1)
                     total # is 32/k5adbit
     conv2bit    --- 2bit data conversion table
     k5adbit     --- K5       AD bits  (1|2|4|8)
     nbit        --- already filled out bits in iwK5 (starting at 0 and <32) 
     iwK5        --- K5 a 32bit word data

  [return]
     nbit        --- already filled out bits in iwK5 (maximum is 32) 
     iwK5        --- K5 a 32bit word data

*/
{
    int m;
    int iw, ik5, nb, ipick;

    int imask[]={0xFFFFFFFE,0xFFFFFFFD,0xFFFFFFFB,0xFFFFFFF7,
                 0xFFFFFFEF,0xFFFFFFDF,0xFFFFFFBF,0xFFFFFF7F,
                 0xFFFFFEFF,0xFFFFFDFF,0xFFFFFBFF,0xFFFFF7FF,
                 0xFFFFEFFF,0xFFFFDFFF,0xFFFFBFFF,0xFFFF7FFF,
                 0xFFFEFFFF,0xFFFDFFFF,0xFFFBFFFF,0xFFF7FFFF,
                 0xFFEFFFFF,0xFFDFFFFF,0xFFBFFFFF,0xFF7FFFFF,
                 0xFEFFFFFF,0xFDFFFFFF,0xFBFFFFFF,0xF7FFFFFF,
                 0xEFFFFFFF,0xDFFFFFFF,0xBFFFFFFF,0x7FFFFFFF};
    int ibset[]={0x00000001,0x00000002,0x00000004,0x00000008,
                 0x00000010,0x00000020,0x00000040,0x00000080,
                 0x00000100,0x00000200,0x00000400,0x00000800,
                 0x00001000,0x00002000,0x00004000,0x00008000,
                 0x00010000,0x00020000,0x00040000,0x00080000,
                 0x00100000,0x00200000,0x00400000,0x00800000,
                 0x01000000,0x02000000,0x04000000,0x08000000,
                 0x10000000,0x20000000,0x40000000,0x80000000};

    int imask2[]={0xFFFFFFFC,0xFFFFFFF3,0xFFFFFFCF,0xFFFFFF3F,
                  0xFFFFFCFF,0xFFFFF3FF,0xFFFFCFFF,0xFFFF3FFF,
                  0xFFFCFFFF,0xFFF3FFFF,0xFFCFFFFF,0xFF3FFFFF,
                  0xFCFFFFFF,0xF3FFFFFF,0xCFFFFFFF,0x3FFFFFFF};

    int imask4[]={0xFFFFFFF0,0xFFFFFF0F,0xFFFFF0FF,0xFFFF0FFF,
                  0xFFF0FFFF,0xFF0FFFFF,0xF0FFFFFF,0x0FFFFFFF};

    int imask8[]={0xFFFFFF00,0xFFFF00FF,0xFF00FFFF,0x00FFFFFF};

    int chmax=1;
    //chmax=nbitstream/adbit;

    iw=iwADS;
    ik5= *iwK5;
    nb=*nbit;


//    if(adsch > chmax){
//       ipick=0;
//    } else {
    if(k5adbit==4){
        m=(nthsample-1)*4;
        ipick= (int)(iw >> m) & 0xF;
    } else  if(k5adbit==8){
        m=(nthsample-1)*8;
        ipick= (int)(iw >> m) & 0xFF;
    } else {
        m=(nthsample-1)*2;
        ipick= (int)(iw >> m) & 0x3;
    }
    
    if(k5adbit==2){  // convert to K5 2bit mode
       ipick=conv2bit[ipick];
       ik5 = (ik5 & imask2[nb/2]) | (ipick << nb);
       nb+=2;
    } else if(k5adbit==4){  // convert to K5 4bit mode
       ik5 = (ik5 & imask4[nb/4]) | (ipick << nb);
       nb+=4;
    } else if(k5adbit==8){  // convert to K5 8bit mode
       ik5 = (ik5 & imask8[nb/8]) | (ipick << nb);
       nb+=8;
    } else {    // pick up 1bit data
       ipick >>= 1;   // MSB get
       ik5 = (ik5 & imask[nb]) | (ipick << nb);
       nb++;
    }
    *nbit=nb;
    *iwK5=ik5;
}

void conv_ads2k5_4ch(int iwADS, int *chtable, int *conv2bit, int k5adbit,
                    int *nbit, int *iwK5)
/*
    ADS3000-DBBC to K5 conversion 4ch mode engine 2010-06-07 

        ADS3000-DBBC format is fixed to as follows

             nbitstream : 32
             ads_adbit  :  2 or 4 or 8
             chmax      : 16

  [in]
     iwADS       --- ADS3000-DBBC   a 32bit word data
     chtable[4]  --- ADS3000-DBBC   ch# to be picked up for K5 4ch data
                        (starting at 1)
                        in case of non-existing ch#, zero data is
                        filled in K5 data
     conv2bit    --- 2bit data conversion table  (only meaningful for 2 bit AD case)
     k5adbit     --- K5       AD bits  (1 or 2 or 4)
     nbit        --- already filled out bits in iwK5 (starting at 0 and <32) 

  [return]
     nbit        --- already filled out bits in iwo (maximum is 32) 
     iwK5        --- K5 a 32bit word data

*/
{
    int m;
    int iw, ik5, nb, ipick;
    int k5ch,adsch;
    int chmax;
    int imask[]={0xFFFFFFFE,0xFFFFFFFD,0xFFFFFFFB,0xFFFFFFF7,
                 0xFFFFFFEF,0xFFFFFFDF,0xFFFFFFBF,0xFFFFFF7F,
                 0xFFFFFEFF,0xFFFFFDFF,0xFFFFFBFF,0xFFFFF7FF,
                 0xFFFFEFFF,0xFFFFDFFF,0xFFFFBFFF,0xFFFF7FFF,
                 0xFFFEFFFF,0xFFFDFFFF,0xFFFBFFFF,0xFFF7FFFF,
                 0xFFEFFFFF,0xFFDFFFFF,0xFFBFFFFF,0xFF7FFFFF,
                 0xFEFFFFFF,0xFDFFFFFF,0xFBFFFFFF,0xF7FFFFFF,
                 0xEFFFFFFF,0xDFFFFFFF,0xBFFFFFFF,0x7FFFFFFF};
    int ibset[]={0x00000001,0x00000002,0x00000004,0x00000008,
                 0x00000010,0x00000020,0x00000040,0x00000080,
                 0x00000100,0x00000200,0x00000400,0x00000800,
                 0x00001000,0x00002000,0x00004000,0x00008000,
                 0x00010000,0x00020000,0x00040000,0x00080000,
                 0x00100000,0x00200000,0x00400000,0x00800000,
                 0x01000000,0x02000000,0x04000000,0x08000000,
                 0x10000000,0x20000000,0x40000000,0x80000000};

    int imask2[]={0xFFFFFFFC,0xFFFFFFF3,0xFFFFFFCF,0xFFFFFF3F,
                  0xFFFFFCFF,0xFFFFF3FF,0xFFFFCFFF,0xFFFF3FFF,
                  0xFFFCFFFF,0xFFF3FFFF,0xFFCFFFFF,0xFF3FFFFF,
                  0xFCFFFFFF,0xF3FFFFFF,0xCFFFFFFF,0x3FFFFFFF};

    int imask4[]={0xFFFFFFF0,0xFFFFFF0F,0xFFFFF0FF,0xFFFF0FFF,
                  0xFFF0FFFF,0xFF0FFFFF,0xF0FFFFFF,0x0FFFFFFF};

    int imask8[]={0xFFFFFF00,0xFFFF00FF,0xFF00FFFF,0x00FFFFFF};
    //int chmax=16;

    iw=iwADS;
    ik5= *iwK5;
    nb=*nbit;
    chmax=16;                // 2014-08-12
    if(k5adbit==4) chmax=8;  // 2014-08-12
    if(k5adbit==8) chmax=4;  // 2014-08-12

    for(k5ch=0; k5ch<4; k5ch++){
       adsch=chtable[k5ch];
       if(adsch > chmax){
          ipick=0;
       } else if(k5adbit==4) {   // 2014-08-12
          m=(adsch-1)*4;
          ipick= (int)(iw >> m) & 0xF;
       } else if(k5adbit==8) {   // 2014-08-12
          m=(adsch-1)*8;
          ipick= (int)(iw >> m) & 0xF;
       } else {
          m=(adsch-1)*2;
          ipick= (int)(iw >> m) & 0x3;
       }
       if(k5adbit==2){  // convert to K5 2bit mode
          ipick=conv2bit[ipick];
          ik5 = (ik5 & imask2[nb/2]) | (ipick << nb);
          nb+=2;
       } else if(k5adbit==4){  // convert to K5 4bit mode    2014-08-12
          ik5 = (ik5 & imask4[nb/4]) | (ipick << nb);
          nb+=4;
       } else if(k5adbit==8){  // convert to K5 8bit mode    2014-08-12
          ik5 = (ik5 & imask8[nb/8]) | (ipick << nb);
          nb+=8;
       } else {    // pick up 1bit data
          ipick >>= 1;   // MSB get
          ik5 = (ik5 & imask[nb]) | (ipick << nb);
          nb++;
       }
    }
    *nbit=nb;
    *iwK5=ik5;
}


void conv_ads2k5_4x4ch(int iwADS, int *chtable, int *conv2bit, int k5adbit,
                    int *nbit, int *iwk5)
/*
    ADS3000-DBBC to K5 conversion 4x4ch mode engine 2010-06-08 

        ADS3000-DBBC format is fixed to as follows

             nbitstream : 32
             ads_adbit  :  2
             chmax      : 16

  [in]
     iwADS       --- ADS3000-DBBC   a 32bit word data
     chtable[16]  --- ADS3000-DBBC   ch# to be picked up for K5 4x4ch data
                        (starting at 1)
                        in case of non-existing ch#, zero data is
                        filled in K5 data
     conv2bit    --- 2bit data conversion table
     k5adbit     --- K5       AD bits  (1 or 2)
     nbit        --- already filled out bits in iwK5 (starting at 0 and <32) 

  [return]
     nbit        --- already filled out bits in iwo (maximum is 32) 
     iwk5[4]     --- K5 a 4x32bit word data for 4 unit

*/
{
    int m,n;
    //int iw, ik5, nb, ipick;
    int iw, nbi, nb, ipick;
    int k5ch,adsch;
    int imask[]={0xFFFFFFFE,0xFFFFFFFD,0xFFFFFFFB,0xFFFFFFF7,
                 0xFFFFFFEF,0xFFFFFFDF,0xFFFFFFBF,0xFFFFFF7F,
                 0xFFFFFEFF,0xFFFFFDFF,0xFFFFFBFF,0xFFFFF7FF,
                 0xFFFFEFFF,0xFFFFDFFF,0xFFFFBFFF,0xFFFF7FFF,
                 0xFFFEFFFF,0xFFFDFFFF,0xFFFBFFFF,0xFFF7FFFF,
                 0xFFEFFFFF,0xFFDFFFFF,0xFFBFFFFF,0xFF7FFFFF,
                 0xFEFFFFFF,0xFDFFFFFF,0xFBFFFFFF,0xF7FFFFFF,
                 0xEFFFFFFF,0xDFFFFFFF,0xBFFFFFFF,0x7FFFFFFF};
    int ibset[]={0x00000001,0x00000002,0x00000004,0x00000008,
                 0x00000010,0x00000020,0x00000040,0x00000080,
                 0x00000100,0x00000200,0x00000400,0x00000800,
                 0x00001000,0x00002000,0x00004000,0x00008000,
                 0x00010000,0x00020000,0x00040000,0x00080000,
                 0x00100000,0x00200000,0x00400000,0x00800000,
                 0x01000000,0x02000000,0x04000000,0x08000000,
                 0x10000000,0x20000000,0x40000000,0x80000000};

    int imask2[]={0xFFFFFFFC,0xFFFFFFF3,0xFFFFFFCF,0xFFFFFF3F,
                  0xFFFFFCFF,0xFFFFF3FF,0xFFFFCFFF,0xFFFF3FFF,
                  0xFFFCFFFF,0xFFF3FFFF,0xFFCFFFFF,0xFF3FFFFF,
                  0xFCFFFFFF,0xF3FFFFFF,0xCFFFFFFF,0x3FFFFFFF};

    int chmax=16;

    iw=iwADS;
    //ik5= *iwK5;
    nbi=*nbit;

    for(n=0; n<4; n++){
        nb=nbi;
        for(k5ch=0; k5ch<4; k5ch++){
           adsch=chtable[n*4+k5ch];
           if(adsch > chmax){
              ipick=0;
           } else {
              m=(adsch-1)*2;
              ipick= (int)(iw >> m) & 0x3;
           }
           if(k5adbit==2){  // convert to K5 2bit mode
              ipick=conv2bit[ipick];
              iwk5[n] = (iwk5[n] & imask2[nb/2]) | (ipick << nb);
              nb+=2;
           } else {    // pick up 1bit data
              ipick >>= 1;   // MSB get
              iwk5[n] = (iwk5[n] & imask[nb]) | (ipick << nb);
              nb++;
           }
        }
    }
    *nbit=nb;
}


void conv_ads2k5_4x4ch_fast(int iwADS,  int k5adbit, int *nbit, int *iwk5)
/*
    ADS3000-DBBC to K5 conversion 4x4ch mode engine 2010-06-08 
       *** fast version (cahannel allocation and 2bit conversion table is fixed) ****

        ADS3000-DBBC format is fixed to as follows

             nbitstream : 32
             ads_adbit  :  2
             chmax      : 16

  [in]
     iwADS       --- ADS3000-DBBC   a 32bit word data
     k5adbit     --- K5       AD bits  (1 or 2)
     nbit        --- already filled out bits in iwK5 (starting at 0 and <32) 

  [return]
     nbit        --- already filled out bits in iwo (maximum is 32) 
     iwk5[4]     --- K5 a 4x32bit word data for 4 unit

*/
{
    int m,n;
    //int iw, ik5, nb, ipick;
    int iw, nbi, nb, ipick;
    int k5ch,adsch;
    int imask[]={0xFFFFFFFE,0xFFFFFFFD,0xFFFFFFFB,0xFFFFFFF7,
                 0xFFFFFFEF,0xFFFFFFDF,0xFFFFFFBF,0xFFFFFF7F,
                 0xFFFFFEFF,0xFFFFFDFF,0xFFFFFBFF,0xFFFFF7FF,
                 0xFFFFEFFF,0xFFFFDFFF,0xFFFFBFFF,0xFFFF7FFF,
                 0xFFFEFFFF,0xFFFDFFFF,0xFFFBFFFF,0xFFF7FFFF,
                 0xFFEFFFFF,0xFFDFFFFF,0xFFBFFFFF,0xFF7FFFFF,
                 0xFEFFFFFF,0xFDFFFFFF,0xFBFFFFFF,0xF7FFFFFF,
                 0xEFFFFFFF,0xDFFFFFFF,0xBFFFFFFF,0x7FFFFFFF};
    int ibset[]={0x00000001,0x00000002,0x00000004,0x00000008,
                 0x00000010,0x00000020,0x00000040,0x00000080,
                 0x00000100,0x00000200,0x00000400,0x00000800,
                 0x00001000,0x00002000,0x00004000,0x00008000,
                 0x00010000,0x00020000,0x00040000,0x00080000,
                 0x00100000,0x00200000,0x00400000,0x00800000,
                 0x01000000,0x02000000,0x04000000,0x08000000,
                 0x10000000,0x20000000,0x40000000,0x80000000};

    int imask2[]={0xFFFFFFFC,0xFFFFFFF3,0xFFFFFFCF,0xFFFFFF3F,
                  0xFFFFFCFF,0xFFFFF3FF,0xFFFFCFFF,0xFFFF3FFF,
                  0xFFFCFFFF,0xFFF3FFFF,0xFFCFFFFF,0xFF3FFFFF,
                  0xFCFFFFFF,0xF3FFFFFF,0xCFFFFFFF,0x3FFFFFFF};

    int chmax=16;

    iw=iwADS;
    //ik5= *iwK5;
    nbi=*nbit;

    for(n=0; n<4; n++){
        nb=nbi;
        for(k5ch=0; k5ch<4; k5ch++){
           //adsch=n*4+k5ch;
           adsch=n*4+k5ch+1;    // 2010-06-21
           if(adsch > chmax){
              ipick=0;
           } else {
              m=(adsch-1)*2;
              ipick= (int)(iw >> m) & 0x3;
           }
           if(k5adbit==2){  // convert to K5 2bit mode
              //ipick=conv2bit[ipick];
              iwk5[n] = (iwk5[n] & imask2[nb/2]) | (ipick << nb);
              nb+=2;
           } else {    // pick up 1bit data
              ipick >>= 1;   // MSB get
              iwk5[n] = (iwk5[n] & imask[nb]) | (ipick << nb);
              nb++;
           }
        }
    }
    *nbit=nb;
}



void conv_ads2k5_2x4ch_fast(int iwADS,  int k5adbit, int *nbit, int *iwk5)
/*
    ADS3000-DBBC to K5 conversion 2x4ch mode engine 2014-08-12 
       *** fast version (cahannel allocation is fixed) ****

        ADS3000-DBBC format is fixed to as follows

             nbitstream : 32
             ads_adbit  :  4
             chmax      :  8

  [in]
     iwADS       --- ADS3000-DBBC   a 32bit word data
     k5adbit     --- K5       AD bits  (1 or 2 or 4)
     nbit        --- already filled out bits in iwK5 (starting at 0 and <32) 

  [return]
     nbit        --- already filled out bits in iwo (maximum is 32) 
     iwk5[2]     --- K5 a 2x32bit word data for 4 unit

*/
{
    int m,n;
    //int iw, ik5, nb, ipick;
    int iw, nbi, nb, ipick;
    int k5ch,adsch;
    int imask[]={0xFFFFFFFE,0xFFFFFFFD,0xFFFFFFFB,0xFFFFFFF7,
                 0xFFFFFFEF,0xFFFFFFDF,0xFFFFFFBF,0xFFFFFF7F,
                 0xFFFFFEFF,0xFFFFFDFF,0xFFFFFBFF,0xFFFFF7FF,
                 0xFFFFEFFF,0xFFFFDFFF,0xFFFFBFFF,0xFFFF7FFF,
                 0xFFFEFFFF,0xFFFDFFFF,0xFFFBFFFF,0xFFF7FFFF,
                 0xFFEFFFFF,0xFFDFFFFF,0xFFBFFFFF,0xFF7FFFFF,
                 0xFEFFFFFF,0xFDFFFFFF,0xFBFFFFFF,0xF7FFFFFF,
                 0xEFFFFFFF,0xDFFFFFFF,0xBFFFFFFF,0x7FFFFFFF};
    int ibset[]={0x00000001,0x00000002,0x00000004,0x00000008,
                 0x00000010,0x00000020,0x00000040,0x00000080,
                 0x00000100,0x00000200,0x00000400,0x00000800,
                 0x00001000,0x00002000,0x00004000,0x00008000,
                 0x00010000,0x00020000,0x00040000,0x00080000,
                 0x00100000,0x00200000,0x00400000,0x00800000,
                 0x01000000,0x02000000,0x04000000,0x08000000,
                 0x10000000,0x20000000,0x40000000,0x80000000};

    int imask2[]={0xFFFFFFFC,0xFFFFFFF3,0xFFFFFFCF,0xFFFFFF3F,
                  0xFFFFFCFF,0xFFFFF3FF,0xFFFFCFFF,0xFFFF3FFF,
                  0xFFFCFFFF,0xFFF3FFFF,0xFFCFFFFF,0xFF3FFFFF,
                  0xFCFFFFFF,0xF3FFFFFF,0xCFFFFFFF,0x3FFFFFFF};
    int imask4[]={0xFFFFFFF0,0xFFFFFF0F,0xFFFFF0FF,0xFFFF0FFF,
                  0xFFF0FFFF,0xFF0FFFFF,0xF0FFFFFF,0x0FFFFFFF};
    int chmax=8;

    iw=iwADS;
    //ik5= *iwK5;
    nbi=*nbit;

    for(n=0; n<2; n++){
        nb=nbi;
        for(k5ch=0; k5ch<4; k5ch++){
           //adsch=n*4+k5ch;
           adsch=n*4+k5ch+1;
           if(adsch > chmax){
              ipick=0;
           } else if(k5adbit==4){
              m=(adsch-1)*4;
              ipick= (int)(iw >> m) & 0xF;
           } else {
              m=(adsch-1)*2;
              ipick= (int)(iw >> m) & 0x3;
           }
           if(k5adbit==2){  // convert to K5 2bit mode
              //ipick=conv2bit[ipick];
              iwk5[n] = (iwk5[n] & imask2[nb/2]) | (ipick << nb);
              nb+=2;
           } else if(k5adbit==4){  // convert to K5 4bit mode
              //ipick=conv2bit[ipick];
              iwk5[n] = (iwk5[n] & imask4[nb/4]) | (ipick << nb);
              nb+=4;
           } else {    // pick up 1bit data
              ipick >>= 1;   // MSB get
              iwk5[n] = (iwk5[n] & imask[nb]) | (ipick << nb);
              nb++;
           }
        }
    }
    *nbit=nb;
}


void k5name_make_fmode(char *adsname, char *outdir, int fmode, int unit, int channel, int adsnumch, 
                 char *k5name)
{
/*
         2010-06-07
         2014-08-12  adsnumch is added
  Create K5 file name 

  Naming rule for K5 file created (ADS : original ADS3000 file name)
    fmode=1
         1ch mode
            ADS.k5-NN ---- where NN is channel number 01-16
         but if adsnumch=1, name is ADS.k5
    fmode=2
         4ch mode
            ADS.k5[a|b|c|d] --- where a|b|c|d for unit=1|2|3|4
                                or omitted for unit=0
    fmode=3
         4x4ch mode
            ADS.k5a ---- for unit#1  (CH 01-04)
            ADS.k5b ---- for unit#2  (CH 05-08)
            ADS.k5c ---- for unit#3  (CH 09-12)
            ADS.k5d ---- for unit#4  (CH 13-16)
    fmode=4
         2x4ch mode
            ADS.k5a ---- for unit#1  (CH 01-04)
            ADS.k5b ---- for unit#2  (CH 05-08)
*/
    int i,len;
    char k5dir[256],dir[256], name[256],fname[256]; 
    char cbuf[256];
    char ag[4][2]={"a","b","c","d"};

    parse_dir_and_name(adsname, dir, name);

    // remove file extension

    len=strlen(name);

    if(adsnumch > 1){  // 2014-08-12
      // find last "." in the name to remove extension
      for(i=len-1; i>=0; i--){
         if(name[i]=='.'){
             name[i]='\0';
             break;
         }
      }
    }


    // out directory prepare
    if(outdir[0] == '\0'){
        k5dir[0]='\0';
    } else {
        // converting '\' to '/' if so
        len=strlen(outdir);
        //printf("C %d %s\n",len,outdir);
        for(i=0; i<len; i++){
            if(outdir[i]=='\\'){
                k5dir[i]='/';
            } else {
                k5dir[i]=outdir[i];
            }
        }
        k5dir[len]='\0';
        // check apri dir name accompanied by "/" or not 
        if(k5dir[len-1] != '/') {
           k5dir[len]='/';
           k5dir[len+1]='\0';
        }
    }
    // create k5name
    strcpy(fname,name);
    strcat(fname,".k5");
    if(fmode == 1){  //  1ch mode
       //if(channel > 0) {
       if(channel > 0 && adsnumch > 1) { // 2014-08-12
           sprintf(cbuf,"-%02d",channel);
           strcat(fname,cbuf);
       }
    } else {
        if(unit > 0 && unit <5) {
           strcat(fname,ag[unit-1]);
        }
    }
    strcpy(k5name,k5dir);
    strcat(k5name,fname);
}


void vssp_hd_make_ini(vssp_head_t *vpb)
{
    int n,fid,aid;
    //long int w1;
    int w1;
    unsigned char b,c;

    vpb->psync2=0x8B;  // K5/VSSP
    if(vpb->format==2) vpb->psync2=0x8C;  // K5/VSSP32

    /* sync pattern 1 set */
    for (n=0; n<4; n++){
       //ipb->hd[n]=0xFF;
       vpb->bhead[n]=0xFF;
    }
    /* seconds set */
    w1=vpb->seconds;
    vpb->bhead[4]=(unsigned char)(w1 & 0xFF);
    vpb->bhead[5]=(unsigned char)((w1 & 0xFF00) >> 8);
    b=(unsigned char)((w1 & 0x10000) >> 16);
    /* ch flag set */
    if( vpb->numch == 4){
        b |= 0x2;
    }
    /* sampling freq index set */
    fid=0;
    //if( ipb->sfreq ==    40) fid=0;
    if( vpb->sfreq ==   100) fid=1;
    if( vpb->sfreq ==   200) fid=2;
    if( vpb->sfreq ==   500) fid=3;
    if( vpb->sfreq ==  1000) fid=4;
    if( vpb->sfreq ==  2000) fid=5;
    if( vpb->sfreq ==  4000) fid=6;
    if( vpb->sfreq ==  8000) fid=7;
    if( vpb->sfreq == 16000) fid=8;
    if( vpb->sfreq == 32000) fid=9;  
    if( vpb->sfreq == 64000) fid=10; 
    if( vpb->sfreq == 128000) fid=11; 
    if( vpb->sfreq == 256000) fid=12;
    if( vpb->sfreq == 512000) fid=13;
    if( vpb->sfreq == 1024000) fid=14;
    if( vpb->sfreq == 2048000) fid=15;
    c=(unsigned char)(fid << 2);
    b |= c;
    /* A/D bit length set */
    aid=0;
    //if( ipb->adbit == 1 ) aid=0;
    if( vpb->adbit == 2 ) aid=1;
    if( vpb->adbit == 4 ) aid=2;
    if( vpb->adbit == 8 ) aid=3;
    c=(unsigned char)(aid << 6);
    b |= c;
    vpb->bhead[6]=b;

    /* sync pattern 2 set */
    vpb->bhead[7]=vpb->psync2;
                // 2nd sync pattern
                //     0x8B for K5/VSSP
                //     0x8C for K5/VSSP32


    /* prepare VSSP32 header portion */
    vpb->bhead[8]=(unsigned char)(vpb->totalday & 0xFF);    // total day lower 8 bit
    vpb->bhead[9]=(unsigned char)((vpb->totalday & 0x100) >> 8 );    // total day MSB
    vpb->bhead[9] |= (unsigned char)(((vpb->year % 1000) & 0x7F) << 1) ;   // year
    vpb->bhead[10]=(unsigned char)(vpb->aux_size & 0xFF);   // aux field size (bytes)
    vpb->bhead[11]=(unsigned char)((vpb->v_major & 0xF) << 4);   // major version
    vpb->bhead[11] = (unsigned char)(vpb->v_minor & 0xF);   // minor version

    /* prepare AUX field VSSP32 */
    /* format field set */
    vpb->bhead[12]= (unsigned char)(vpb->aux_fmt & 0xFF);
    /* filter field set */
    vpb->bhead[13]= (unsigned char)(vpb->filter & 0xFF);
    /* aux field set */
    if(vpb->aux_fmt==1){
        /* station field set */
        memcpy(vpb->bhead+14,vpb->aux_sid2,2);  // station ID
        memcpy(vpb->bhead+16,vpb->aux_station,8);  // station name
        memcpy(vpb->bhead+24,vpb->aux_hostname,8);  // Host PC name
    } else if(vpb->aux_fmt==2){
        /* fill out data with 0x55 */
        memset(vpb->bhead+14,0x55,10);
        /* hostname field set */
        memcpy(vpb->bhead+24,vpb->aux_hostname,8);  // Host PC name
    } else if(vpb->aux_fmt==85){
        /* fill out all data with 0x55 */
        memset(vpb->bhead+14,0x55,18);
    } else if(vpb->aux_fmt==170){
        /* fill out all data with 0xAA */
        memset(vpb->bhead+14,0xAA,18);
    } else {
       /* format 0 is adapted */
       /* fill out all data with 0x00 */
        memset(vpb->bhead+12,0x00,20);
    }
}

void vssp_hd_make(IPBOARD_T *ipb)
{
    /* IP-VLBI{[ho̓f[^wb_[̂ݍXV */
    //long int w1;
    int w1;
    unsigned char b,c;
    /* seconds set */
    w1=ipb->seconds;
    b=(unsigned char)((w1 & 0x10000) >> 16);
    ipb->hd[4]=(unsigned char)(w1 & 0xFF);
    ipb->hd[5]=(unsigned char)((w1 & 0xFF00) >> 8);
    b=ipb->hd[6];
    c=(unsigned char)((w1 & 0x10000) >> 16);
    b=(b & 0xFE) | (c & 0x1);
    ipb->hd[6]=b;
}
