/*************************************************************************/
/*                                                                       */
/*   Program Name :  m5b_recov                                           */
/*                                                                       */
/*     Recover Mark-5B format file which is generated by k5tom5b         */
/*        of which Ver.2014-06-11 or before                              */
/*                                                                       */
/*      Version:  1.00  2016-07-20                                       */
/*                                                                       */
/*  copyright (c) 2016-     T.Kondo All Right Reserved                   */
/*                                                                       */
/*************************************************************************/
/*

    Mark5B format file converted from K5 file by using "k5tom5b" (Ver.2014-06-11 or before)
    has a ploblem in time label when there is a day boundary during a scan.
    "m5b_recov" corrects this problem. 
    

          T.Kondo 2016-07-20 start development


    m5b_recov m5bfile [outfile]
    
     where   m5bfile  -- Mark5B file name
             outfile  -- output file name
                         default is m5bfile + '.recov'
                    
*/
#define PROGRAM_VERSION "m5b_recov (Ver 1.00  2016-07-20)"
#include "m5b_recov.h"


/* environment variable for Mark5 data default directory set */
#define M5DIR "M5DIR" // the environment variable for Mark5 data default directory


void param_get_m5b_recov(int argc, char *argv[], m5b_recov_t *m5b)
{
    //1st param should be mark5 name
    if (argc>=2){
        strcpy(m5b->m5bname,*(argv+1));
    } else {
        m5b->m5bname[0]='\0';
    }

    if (argc>=3){
        strcpy(m5b->outfile,*(argv+2));
    } else {
        m5b->outfile[0]='\0';
    }

}

void print_usage(void)
{
    //printf("m5tok5Rnp  %s\n",M5TOK5RNP_VERSION);
    printf("%s\n",PROGRAM_VERSION);
    printf("Usage: m5b_recov m5bfile [outfile]\n");
    printf("\n");
    printf("where   m5bfile -- Mark5B data file name\n");
    printf("        outfile -- output file name\n");
    printf("                   (default: m5bfile+\".recov\")\n");
    printf("\n");
}


int main(int argc, char *argv[])
{
       int iret;
       char program_version[60];
       m5b_recov_t m5b;    

       if(argc == 1 || (argc == 2 && strncmp(*(argv+1),"--help",6)==0 )){ // 2008-02-19
          print_usage();
          exit(0);
       }
       if(argc == 2 && strncmp(*(argv+1),"--ver",5)==0 ){ // 2008-02-19
          printf("\n");
          printf("%s\n",PROGRAM_VERSION);
          printf("\n");
          exit(0);
       } 
       if (argc>=2){
          if(strncmp(*(argv+1),"env",3)==0){
          printf("   Enviroment variables\n");
          printf("        %s ---   default directory for Mark5 data\n",M5DIR);
          printf("                      ( %s )\n",getenv(M5DIR));
          printf("\n");
          exit(0);
          }
       }

       strcpy(program_version,PROGRAM_VERSION);

       /* Program Name Display */
       printf("****************************************************\n");
       printf("*    edit Mark-5B data time                        *\n");
       printf("*      %s            *\n",PROGRAM_VERSION);
       printf("*        Kashima Space Technology Center/NICT      *\n");
       printf("****************************************************\n");
       printf("\n");

       /* run parameter get */
       param_get_m5b_recov(argc, argv, &m5b);

       // outname creation
       if(m5b.outfile[0]=='\0'){
           strcpy(m5b.outfile,m5b.m5bname);
           strcat(m5b.outfile,".recov");
       }

       iret=mk5b_dat_anal_edit(&m5b);

	   if(iret == 1){  // this is Mark5B format
		   printf("\n");
		   printf("********************** SUMMARY of DATA EDIT ********************\n");
		   printf(" Original Data File Name   : %s\n",m5b.m5bname);
		   printf(" Data File Name after edit : %s\n",m5b.outfile);
		   //printf(" Data Format : Mark-5B\n");
           //printf(" File Size   : %ld bytes",fsize);       
           printf(" File Size   : %lld bytes",(long long)m5b.fsize);       
           printf("    Total # of Frames: %f\n",m5b.tframe);  // int64 type variable and other typ in 1 line
                                                           // results in 0 value for 2nd one in WIN32
                                                           // So, seprated in 2 lines
		   printf(" 1st Header Information\n");
		   printf("   Time (MJD HH:MM:SS.SSSS) : %03d %02d:%02d:%02d.%04d\n",
			   m5b.istart[1],m5b.istart[2],m5b.istart[3],m5b.istart[4],m5b.istart[5]);
		   printf(" Last Header Information\n");
		   printf("   Time (MJD HH:MM:SS.SSSS) : %03d %02d:%02d:%02d.%04d\n",
			   m5b.istop[1],m5b.istop[2],m5b.istop[3],m5b.istop[4],m5b.istop[5]);
		   printf("   Frame # (in a second)    : %10d\n",m5b.framstart);
		   if(m5b.istart[5]==0 && m5b.framstart !=0){  // ".SSSS" field not correct   2012-11-26
			   printf("     !!! Warning!! "".SSSS"" field in start time may be incorrect! !!!\n");
			   m5b.istart[5]=(int)(((double)m5b.framstart*1.0/(double)m5b.framsec)*10000.0);
			   printf("     !!! Start time supposed from 1st frame# is %02d:%02d:%02d.%04d  !!!\n",
				   m5b.istart[2],m5b.istart[3],m5b.istart[4],m5b.istart[5]);
		   }
		   printf("   User Specified (16 bits) : %04X\n",(unsigned short int)m5b.iuid);
		   printf("   TVG data flag            : %d\n",m5b.tvgflag);
		   printf(" Data Rate (Mbps) : %6.1f\n",m5b.fbps/1.0e6);
           printf(" Frames/sec       : %d\n",m5b.framsec);       
           printf(" Data Length (sec): %f\n",m5b.dlength);       
           printf("****************************************************************\n");
		   exit(0);
       } else {

	      printf("  ==> Not Mark5B format\n");
       }

       return 0;
       
}




int mk5b_dat_anal_edit(m5b_recov_t *m5b)
{
/*
     Mark5B data file structure analysis and time data recover
     modified from mk5b_dat_anal
     2016-07-20

    [in]
	  m5bname  ---  Mark5 data file name
      outfile  ---  output file name
	[out]
	  istart[6] ----- 1st sync block time  
                         -1,mjd(3digits),hh,mm,ss,msec*10  for VLBA format
	  framstart ----- 1st sync block frame #    (added on 2012-11-26)
	  uid       ----- User specified data (lower 16bits are valid)
	  tvgflag   ----- TVG flag (1: TVG data  0: real data)
	  fbps      ----- Supposed data rate (bps)
      framsec   ----- frames/sec
      fsize     ----- file size
      tframe    ----- total number of frames (double) (computed from file size)
      dlength   ----- data length in seconds
	  badframes ----- number of bad frames (added on 2012-10-17)
	  totalframes ----- number of frames actually counted (added on 2012-10-17)

    [return]
       0   ---  This is Mark 5B format
       -1  ---  This is not Mark 5B format

*/
       int i,nkai,on_sec_flag,p_flag,p_flag_old,nframeold;
       int nmax;
       unsigned int ibuf[2504];  // for 1 Frame data 2504 X 32 bits
	   unsigned int *head;   // for header 4 X 32 bits
	   unsigned int *idata;  // for data field 2500 X 32 bits
       unsigned int head_old[4];
       int frame32=2500;   // # of 32-bit words in a Mark5B frame
       int nbuf=2504;   //  frame length including header (in 32-bit word)
       int iread,iwrite; 
       int itime[6];
       int seconds,s_seconds,mjd3, s_mjd3,iword;
       int msec10,msec10old, samsec10;
	   unsigned short uid, icrc16;
	   int iok;
	   int nframe;
	   int nframe_max;
       int frame_c; // frame # counter
	   int bframe_c; // bad frame # counter 2012-12-17
	   int tvg;
	   int nmaxhere;
       int mk5B_mode;   // =1  Mark 5B format   =0 No
       double bps, mbps;  // supposed data rate
	   double dtime,dtime0,dt;
	   double dtime1;
	   int n_flag;
	   int bad_frame_flag;
	   int bad_frame_flag_old;
	   int flag_frac_sec;  // flag (set 1 if fractional second set correctry)
	   int nfram_per_sec;  //  true # of frames per sec
       char cbuf[10];
       int64_t dfsize;
       FILE *fin, *fo;

       mk5B_mode=1;
	   head=ibuf;
	   idata=ibuf+4;

       //on_sec_flag=mode/10;    // 0 for every frame   1 for every second
       //d_mode=mode-10*on_sec_flag;
       on_sec_flag=1;
       strcpy(cbuf,"FRAME#");
       if(on_sec_flag==1) strcpy(cbuf,"  SEC#");

       p_flag=0;
       p_flag_old=0;
	   bad_frame_flag_old=0;
	   flag_frac_sec=0;
	   nfram_per_sec=0;
	   nframe_max=0;

       nmax=0;

	   nmaxhere=nmax;

       iok=0;
       /* Data file open */
       if( (fin  = fopen( m5b->m5bname, "rb" )) == NULL )
       {
           printf("Data File %s   Open Error! \n",m5b->m5bname);
		   exit(1);
       }
       printf("Data File : %s\n",m5b->m5bname);
       dfsize=get_file_size(m5b->m5bname);
       m5b->fsize=dfsize;
       m5b->tframe=(double)dfsize/(double)(nbuf*4);

       /* Output file open */
       if( (fo  = fopen( m5b->outfile, "wb" )) == NULL ){
           printf("  Output File  %s  Open Error! \n",m5b->outfile);
           goto exitreturn;
       }

       //printf("Now analizing the data file is Mark5B or not .....\n"); 

	   nkai=0;
       nframe=0;
       nframeold=0;
       frame_c=0;
	   bframe_c=0;
	   n_flag=0;
	   dtime1=0.0;
       msec10=0;
       msec10old;
       samsec10=0;

	   dtime0=0.0;



       while(1){
          iread = fread( ibuf, sizeof( unsigned int ), nbuf, fin );  // 1 frame read
          if (iread ==0 ) { // EOF
			 //printf("EOF found\n");
             goto exitreturn; 
          }
          frame_c++;  // total frame number counter

          nframeold=nframe;
          p_flag=0;

		  bad_frame_flag=0; // 2012-10-17

          if(mk5b_sync(head, itime, &seconds, &uid, &tvg, &nframe,&icrc16) < 0){
              //printf("not Mark5B header\n");
              iok=-1;
              goto exitreturn;
          }

          memcpy(head_old,head,4*sizeof(int));

          if (nkai ==0) mk5B_mode=1;

	      if(nframe > nframe_max) nframe_max=nframe;

          if (nkai ==0) {
              //printf("\n OK this is Mark 5B Format data\n");
			  //printf("\n");
              for (i=0; i<6; i++) m5b->istart[i]=itime[i];
              s_mjd3=itime[1];
              s_seconds=seconds;
			  m5b->framstart=nframe;
			  m5b->iuid=(int)uid;
			  m5b->tvgflag=tvg;



				 //printf("                           TIME CODE   CRC  \n");
				 //printf("%s   SYNC   UUUUTFFF MJDsssss.ssss  16  MJD HH:MM:SS.SSSS   Mbps\n",cbuf);
				 printf("            OLD                  EDITED\n");
				 printf("         TIME CODE   CRC      TIME CODE   CRC\n");
				 printf("%s MJDsssss.ssss  16 -> MJDsssss.ssss  16  MJD HH:MM:SS.SSSS   Mbps\n",cbuf);
		  }

          // time data edit
          if(seconds >= 86400) {
              mjd3=s_mjd3+1;
              seconds-=86400;
              convmjsdsec_word(mjd3,seconds,&iword);
              head[2]=iword;
              m5b_hd_crcc_setup(head);
              if(mk5b_sync(head, itime, &seconds, &uid, &tvg, &nframe,&icrc16) < 0){
                 //printf("not Mark5B header\n");
                 iok=-1;
                 goto exitreturn;
              }
          }

          for (i=0; i<6; i++) m5b->istop[i]=itime[i];
          
          // data copy

          iwrite=fwrite(ibuf,sizeof(int),nbuf,fo);

       
          if(on_sec_flag ==1){
              if(nframe==0) {
                  p_flag=1;
                  nkai++;
              }
			  if(bad_frame_flag==1) p_flag=1;  // 2012-10-18
          } else {
             p_flag=1;
		     nkai++;
          }

		  dtime=((double)itime[5])/10000.0;
          msec10=itime[5];
		  if(msec10 !=0) flag_frac_sec=1;  // detection of fractional second increments 2012-11-26 
		  if(frame_c > 1) {
			  if(bad_frame_flag == 0 && bad_frame_flag_old ==0){
		         dt=dtime-dtime0;
                 samsec10=msec10-msec10old;  //2011-01-12
                 if(samsec10 < 0) samsec10+=10000; //2011-01-12
			     if(frame_c==2 && samsec10 < 3){ // 2011-01-12    if time label less than 0.3 msec
                                                            // rate estimation impossible by using
                                                            // only a frame data (this correspond to
                                                            // 512 Mbps and higer
				     n_flag=1;   // set no calc flag (doesn't calc bps by every frame)
				     dtime1=dtime;


			     }
		         if(dt <= 0.0) dt+=1.0;
		         bps=(double)frame32*32.0/dt;
			  } else { // 2012-10-17
				  bps=0.0;
			  }
		  } else {          // 2011-01-12
			 bps=0.0; // because unable to estimate for first
		  }
		  mbps=bps/1.0e6;

		  /* bps cleaner */
		     // Mark5B time label does not have sufficient digit
		     // so bps is sometimes not correct
		     // now only 2^N Mbps starting from 4 Mbps is assumed
		     // as corrected value

		  if(mbps > 0.0){
			  mbps=pow(2.0, (double)((int)(log(mbps)/log(2.0) +0.5)));
			  bps=mbps*1.0e6;
		  }

		  dtime0=dtime;
          msec10old=msec10; // 2011-01-12

		  bad_frame_flag_old=bad_frame_flag; // 2012-10-17

          /* print header info */
          //if(nkai > 1) {
          if(frame_c > 1) {
			  //if(nkai==2) *fbps=bps;
			  if(n_flag==0){ // 2011-01-12
                 if(frame_c==2) m5b->fbps=bps;
                 if(p_flag_old==1){
					 if(bps != 0.0){ // 2012-10-17
			             printf(" %6.1f",bps/1.0e6);
                         //       printf(" %5d",syncpos);
                         printf("\n");
					 } else {    // 2012-10-17
                        printf("   N.A.\n");
					 }
                 }
			  } else {
                 if(p_flag_old==1){  // 2012-12-03
					 printf("   N.A.\n");
				 }
			  }
		      //if ((nmax > 0) && (nkai > nmax)) goto exitreturn;
		      if ((nmaxhere > 0) && (nkai > nmaxhere)) goto exitreturn;  // 2011-01-12
		  }
          if(p_flag==1){
			  if(on_sec_flag==1) {        // 2012-11-23
                   printf(" %05d ",nkai);
			  } else {
                 printf(" %05d ",nframe);
			  }
			  for (i=2; i<4; i++){         
				 //printf("%08X ",rvsint(head[i]));
				 printf("%08X ",head_old[i]);
			  }
              printf("   ");
			  for (i=2; i<4; i++){         
				 //printf("%08X ",rvsint(head[i]));
				 printf("%08X ",head[i]);
			  }
              printf(" %03d %02d:%02d:%02d.%04d",
                        itime[1],itime[2],itime[3],itime[4],itime[5]);
          }
          p_flag_old=p_flag;
		  //if ((nmax > 0) && (nkai >= nmax)) goto exitreturn;
	   }
exitreturn:
       i=fclose(fin);
       i=fclose(fo);
       m5b->framsec=(int)(m5b->fbps/((double)frame32*32.0) +0.5);
       m5b->dlength=(m5b->tframe)/(double)(m5b->framsec);   // this is approximation of data second length
                                                // because header portion is included
	   m5b->totalframes=frame_c;
	   m5b->badframes=bframe_c;

       return mk5B_mode;
}


int mk5b_sync(int *head, int *itime, int *seconds, unsigned short int *uid,
					  int *tvgflag, int *nframe, unsigned short int *icrc16)
/*
     Mark5B sync detection and analysis
     2016-07-20  

  [in]
      head  ---- pointer to headder array (4 X 32 bits) buffer 

  [out]
      itime[6] --  time in header
                   -1,mjd(3digits),hh,mm,ss,msec*10  (like a VLBA format)
	  uid      --   User-specified field  (16-bit integer)
	  tvgflag  --   Tvg data flag   1: tvg data  0: not tvg data
	  nframe   --   Frame# within second (starting at 0)
	  icrc16   --   CRCC for time field  (16-bit integer)

  [return]
      -1  :   not Mark5B header
       0  :   Mark5B header


*/
{
    int iok;
    int mjd3;
    int sync_pat=0xABADDEED;   // Mark5B sync pattern
    

	iok=-1;


    /* sync detection */
	if( head[0]==sync_pat ){ //  sync detected
	   iok=0;

		/* retrieve user-specified field */
       
		*uid= (unsigned short int)( (head[1] & 0xFFFF0000) >> 16);
		   
		/* retrieve Tvg flag */

		*tvgflag=0;

		if( (head[1] & 0x8000) != 0) *tvgflag=1;

		/*  retrieve frame# */

		*nframe = (int)( head[1] & 0x7FFF);

		/* retrieve time information */

		conv2time_m5b(head+2, itime, &mjd3, seconds);

		/* retrieve CRCC */

		*icrc16= (unsigned short int)(head[3] & 0xFFFF);
	}
    return iok;
}


void gene_updated_time(int *itime, double p_frame)  // 2012-10-17
/*
    generate time increased by p_frame   2012-10-17
*/
{
	double tsec;
	tsec=3600.0*(double)itime[2]+60.0*(double)itime[3]+(double)itime[4]
	      +(double)itime[5]/10000.0;
    tsec+=p_frame;
    // hour 
    itime[2]=(int)tsec/3600;
	if(itime[2] >= 24){
		itime[2]-=24;
		itime[1]++;
	}
    //minute
    itime[3]=((int)tsec % 3600)/60;
    // sec get 
    itime[4]=(int)tsec % 60;
    // msec10 get 
    itime[5]=(int)(fmod(tsec,1.0)*10000.0);
}



/* crc subroutine ripoff  by David*/
int crcc(unsigned char* idata, int len, int mask, int cycl) {
	unsigned int istate = 0;
	int idbit;
	int q, ich, icb;

	for (idbit = 1; idbit <= len; idbit++) {
		q = istate & 1;
		ich = (idbit - 1) / 8;
		icb = 7 - (idbit - 1) % 8;
		if ((((idata[ich] >> icb) & 1) ^ q) == 0)
			istate &= -2;
		else {
			istate ^= mask;
			istate |= 1;
		}
		istate = (istate >> 1) | ((istate & 1) << (cycl -1));  
	}

	return ((int) istate);
}


void conv2time_m5b(int *idat, int *itime, int *mjd3, int *seconds) 
// Mark5B header decode to time       2016-07-21
{
    //int i1,i2,i3,i4,i5, mjd3, seconds, msec10;  // msec10=msec*10
    int i1,i2,i3,i4,i5, mjd, secs, msec10;  // msec10=msec*10
    // lower 3 digit mjd get
    i1=(idat[0] >> 28) & 0xF;
    i2=(idat[0] >> 24) & 0xF;
    i3=(idat[0] >> 20) & 0xF;
    mjd=i1*100+i2*10+i3;

    // integer seconds in day get
    i1=(idat[0] >> 16) & 0xF;
    i2=(idat[0] >> 12) & 0xF;
    i3=(idat[0] >>  8) & 0xF;
    i4=(idat[0] >>  4) & 0xF;
    i5= idat[0] & 0xF;
    secs=i1*10000+i2*1000+i3*100+i4*10+i5;


    // msec*10 get
    i1=(idat[1] >> 28) & 0xF;
    i2=(idat[1] >> 24) & 0xF;
    i3=(idat[1] >> 20) & 0xF;
    i4=(idat[1] >> 16) & 0xF;
    msec10=i1*1000+i2*100+i3*10+i4;

    // 1 digit year set to -1 to notice this is VLBA format
    itime[0]=-1;
    // 3 digit MJD set
    itime[1]=mjd;
    // hour 
    itime[2]=secs/3600;
    //minute
    itime[3]=(secs % 3600)/60;
    // sec get 
    itime[4]=secs % 60;
    // msec10 get 
    itime[5]=msec10;

    *mjd3=mjd;
    *seconds=secs;
}

void convmjsdsec_word(int mjd3, int seconds, int *iword)
{
    int iw,i1,i2,i3,i4,i5;
    
    iw=0;
    // lower 3 digit mjd set
    i1=mjd3/100;
    i2=(mjd3 % 100)/10;
    i3=(mjd3 % 10);
    iw=i1 << 28;
    iw |= (i2 << 24);
    iw |= (i3 << 20);


    // integer seconds in day set
    i1=seconds/10000;
    i2=(seconds % 10000)/1000;
    i3=(seconds % 1000)/100;
    i4=(seconds % 100)/10;
    i5=(seconds % 10);

    iw |= (i1 << 16);
    iw |= (i2 << 12);
    iw |= (i3 <<  8);
    iw |= (i4 <<  4);
    iw |=  i5;

    *iword=iw;
}

void m5b_hd_crcc_setup(int *head)
/*
     Mark5B header block CRCC setup
     2016-07-21 

  [in]
      head     --- Mark5B header (Time field should be already set up)
  [out]
      head     --- Header after CRCC is update
*/
{
    int n;
    int iw;
    int crc16;
    int idat[12];
    unsigned char bdat[6];



    /* Time field */

    //  JJJSSSSSssss

    iw=head[2];
    for(n=0; n<8; n++){
        idat[n]=(iw >> (28-4*n)) & 0xF;
    }
    iw=head[3];
    for(n=0; n<4; n++){
        idat[8+n]=(iw >> (28-4*n)) & 0xF;
    }

    /* CRC16 */   
    for(n=0; n<6; n++){
        bdat[n]= (idat[2*n] << 4) + idat[2*n+1];
    }
	crc16 = crcc(bdat, 48, 040003, 16);
    //printf("Debug CRC16 %08X\n",rvsint(crc16));
    //printf("Debug CRC16 %08X\n",crc16);

	// Mark5B bit order is reversal 
	crc16=rvsint(crc16) >> 16;

    head[3] &= 0xFFFF0000;

	head[3] |= (crc16 & 0xFFFF);
    
}

int64_t get_file_size(char *fname)
{
    /*  get file size opened */
	int result;
#ifdef WIN32                       //2006-09-20
    struct _stati64 info;    // for VC++    
    result=_stati64(fname,&info);  // for VC++
#else
    struct stat info;
    result=stat(fname,&info);
#endif
	if(result != 0) return (int64_t)(-1);
	else
    return (int64_t)info.st_size;
    // use %lld to print out whole number that has int64_t type
}


int rvsint(int i4)
{
    // reverse integer narabi
    int iw, iwork, ipick,i;
    iw=i4;
    iwork=0;
    for(i=0; i<32; i++){
        iwork=iwork << 1;
        ipick=iw & 0x1;
        iwork= ipick | (iwork & 0xFFFFFFFE);
        iw=iw >> 1;
    }
    return iwork;
}
