/**************************************************************/
/*                                                            */
/*   File name :  libskdvex.h                                 */
/*                                                            */
/*      Version:   2018-02-02                                 */
/*                                                            */
/*   Copyright (c) 2005-2018 T.Kondo/NICT All Right Reserved  */
/*                                                            */
/**************************************************************/
/*


2018-02-02
   "ul_by_stat" and "freq_by_stat" were added as members of sked_data_t type variable
2017-09-03
   "setup_ch_alloc_table_by_station" was added
2017-09-01
   "ch_by_stat" was added as a member of sked_data_t type variable
2017-08-28
   maximum number of characters of star name is changed from 11 to 15
2016-10-12
   "pickup_source" and "pickup_baseid" were added to sked_data_t type variable
2016-09-16
   large scale modification
2016-09-14
   definition of MAXPCAL_B, MAXBBC_B, and MAXIF_B was added
2016-08-15
   "adbit[MAXCH]" is added as an member of code_ele_t type variable
2016-08-10
   MAXCH is newly added
2013-01-30
   A member codes_t "code" was added in sked_data_t type variable
   "code_ele_t" and "codes_t" type variable were added.
    functions "TrimTK", "pick_up_sked_codes_section", "readskd_codes" were
	added to reflect the multiple declaration of frequency tables in $CODES
	section in a SKED type schedule file.
2009-11-19
  New members were added to skede_data_t type variable and vex_t type variable.
  "readskd_sked", "readskd_source", "readskd_station", "readskd",
  "SITE_ana", "STATION_ana", "SKED_ana", "SOUECE_ana", "DAS_ana", and
  "vex_read" were updated to know the true number of stations, 
  sources, scans, and data ack terminals in a schedule file even though
  the case of exceeding their allowded maximum defined in src/chead.h, 
  i.e.,  MAXSTAT, MAXSTAR, MAXOBS, and MAXDAS.
  das[20], a member of station_def_t type variable, was changed to
  das[MAXDAS][20].
  Functions "sub_site_ana", "sub_station_ana", "sub_source_ana",
  "sub_sked_ana", and "sub_das_ana" were added.
2009-11-13
  Member of vex type variable "das_def_t das[40]" was changed to 
  "das_def_t das[MAXDAS]" to easily reflect the change og maximum number 
  of DAS  
2009-11-12
  Type 6 naming rule was added in "scan_file_naming"
2009-06-19
  "SITE_ana" was updated not to misunderstand a key word "site_position_epoch"
  to "site_position".
2007-09-26
  new function "parseSBinfo" was added to analyze sideband info
  in a SKED file
2007-04-17
  consider in case of C++ 
2006-05-09
  das_def_t das[20] in vex type structure was changed to das_def_t das[40]
2006-03-17
  new member (usb) was added to schedule type variable to set
  sideband in case of SKED file
2005-07-25
  To reflect "tape early start parameter" in SCHED and VEX,
  das_def_t type was introduced and sked_data_t and vex_t types were 
  modified. Moreover DAS_anal and DAS_mon functions were added and
  vex_read function was updated.  
  Readskd_param function was added and readsked fucntion was updated.
  (line_parse function was also added.)
2005-06-12
  libsked.h and libvex.h are merged into libskdvex.h.
  libskdvex.h is divided into libskdvex.h and libskdvex.c

*/
#ifndef _LIBSKDVEX_
#define _LIBSKDVEX_

#include "chead.h"
#include "libipvlbi.h"
#include <math.h>
#include <ctype.h>

#ifndef MAXCH
//#define MAXCH 16             // Maximum number of channels in a SKED file added on 2016-08-10 
#define MAXCH 32             // Maximum number of channels in a SKED file added on 2016-08-10 
//#define MAXCH 64             // Maximum number of channels in a SKED     2018-02-08 
#endif

/*
#ifndef MAXCODES             // added on 2013-01-30  (usually declared in chead.h)
#define MAXCODES 10          // Maximum number of code (frequency tables) in a SKED schedule file
#endif


#ifndef MAXPCAL_B
#define MAXPCAL_B 20       // Maximum number of PCAL block in VEX file   added on 2016-09-14
#endif

#ifndef MAXBBC_B
#define MAXBBC_B 20        // Maximum number of BBC block in VEX file   added on 2016-09-14
#endif

#ifndef MAXIF_B
#define MAXIF_B  30        // Maximum number of IF block in VEX file   added on 2016-09-14
#endif
*/

#ifndef MAXCODES             // added on 2013-01-30  (usually declared in chead.h) // alived on 2017-08-28
#define MAXCODES 10          // Maximum number of code (frequency tables) in a SKED schedule file
#endif

#ifndef MAX_SRC_NAME      // maximum number of characters of source name  2017-08-28
//#define MAX_SRC_NAME 11   // before 2017-08-28
#define MAX_SRC_NAME 15
#endif

#ifndef MAX_STAT_NAME      // maximum number of characters of station name  2017-08-28
#define MAX_STAT_NAME 11
#endif

extern void ddd2md(int year, int ddd, int* month, int* mday);

typedef struct obs_file_naming
{   // file naming input type variable
     int naming_type;       // naming type;
     char expcode[11];     // experiment code
     char statid[4];       // station ID  A1 or A2
     char statid2[4];      // station ID  A2
     char scanid[20];      // scan ID
     char freqg[4];        // frequency group ID a|b|c|d
     char pcid[4];         // PC ID  '1'|'2'|'3'|'4'

     int numch;   // # of Ch  (1 or 4)
     int obsnum;  // current obs# (1,2,3....)

     //int year;    // 4 digit year
     //int dddf;   // totalday at 1st scan
     //int ddd;    // totalday at current scan
     //int hh;     // hour at the start of current scan
     //int mm;     // minutes at the start of current scan
     //int ss;     // second at the start of current scan
     timeval_k5_t tf;  // time info at 1st scan
     timeval_k5_t tn;  // time info at current scan

     
} obs_file_naming_t;

    /* data structure for $CODES section in a SKED file data (added on 2013-01-28) */
typedef struct code_ele      // added on 2013-01-28
{
     //char statnam[MAXSTAT][11];       // station name
     //char (*statnam)[11];       // station name
     char (*statnam)[MAX_STAT_NAME];       // station name  // 2017-08-28
	 int nstation;                    // number of stations in statnam table
     //double freqtab[MAXCH];         // RF frequency table (Hz)
     double *freqtab;         // RF frequency table (Hz)
     //double pcalf[10][MAXCH];      // PCAL frequency table (Hz)  modified on 2003-07-18
     double *pcalf[10];      // PCAL frequency table (Hz)  modified on 2003-07-18
     //double vbw[MAXCH];         // Video width table (Hz)
     double *vbw;         // Video width table (Hz)
     //char sideband[MAXCH][4];       // side band  U|L or USB|LSB
     char (*sideband)[4];       // side band  U|L or USB|LSB
     int nfreq;                  // number of frequency
     //int adbit[MAXCH];           // AD resolution   added on 2016-08-15
     int *adbit;           // AD resolution   added on 2016-08-15
} code_ele_t;

typedef struct codes       // added on 2013-01-28
{
	 //code_ele_t ctab[MAXCODES];
	 code_ele_t *ctab;    // 2016-09-16
	 int ncode;                  // number of codes
	 int ncode_max;             // maximum number of codes  2016-09-16
	 int usb;  // compulsory sideband set flag    0: set to LSB  (if 0 is set, all will be lsb9
	 double fsample;   // sampling frequency (Hz)     if 0.0, no sampling freq declared
     int nstation_max;           // MAXSTAT maximum number of stations for malloc   2016-09-15
     int ch_max;                // MAXCH maximum number of channels (frequencies) for malloc 2016-09-15
} codes_t;

    /* data structure for SKED (DRUDG) file data */
typedef struct schedule
{
     char expcode[11];           // experiment code

     /* element size MAXSTAR */
     //char (*starnam1)[11]; // star name1    MAXSTAR*11
     //char (*starnam2)[11]; // star name2
     char (*starnam1)[MAX_SRC_NAME]; // star name1    MAXSTAR*MaX_SRC_NAME     2018-08-28
     char (*starnam2)[MAX_SRC_NAME]; // star name2                             2018-08-28
     double *ra;         // star position RA  in degree
     double *dec;        //       "       DEC in degree
     double *epo;        //       "       Epoch
     int    nstar;               // number of stars (maximum is MAXSTAR)
     int    nstar_max;           // MAXSTAR maximum number of stars for malloc  2016-09-15
     int    nstar_true;          // number of stars truely included in schedule file 2009-11-19

     /* element size MAXSTAT */
     //char (*statnam)[11];       // station name
     char (*statnam)[MAX_STAT_NAME];       // station name   // 2017-08-28
     //double xyz[MAXSTAT][3];          // station position (x,y,z) in m
     double (*xyz)[3];          // station position (x,y,z) in m
     char (*statid)[3];         // 1 char or 2 char station ID 
     char (*statid2)[3];         // 1 char or 2 char station ID 
     int nstation;               // number of stations (maximum is MAXSTAT)
     int nstation_max;           // MAXSTAT maximum number of stations for malloc   2016-09-15
     int nstation_true;          // number of stations truely included in schedule file 2009-11-19

     /* element size MAXCH */
     //double freqtab[16];         // RF frequency table (Hz)
     //double pcalf[10][16];      // PCAL frequency table (Hz)  modified on 2003-07-18
     double *freqtab;         // RF frequency table (Hz)    2016-08-10
     //double pcalf[10][MAXCH];      // PCAL frequency table (Hz)   2016-08-10
     double *pcalf[10];      // PCAL frequency table (Hz)   2016-08-10
                                //  up to 10 th PCAL frequency
     char (*sideband)[4];       // side band  U|L or USB|LSB  2016-08-10


     int ch_max;                // MAXCH maximum number of channels (frequencies) for malloc 2016-09-15
     int nfreq;                  // number of frequency

     /* element size MAXCH*MAX_STAT */

     int *ch_by_stat;             // pick up channel allocation table by station 
                                  // pick up channel is defined by position in 32bit
                                  // in case of 1 bit AD   1 to 32
                                  // in case of 2 bit AD   1 to 16
                                  // argument is CH# in VEX file  i.e.,  &CH01 etc
     int *ul_by_stat;             // USB(1) or LSB(0) table by station  2018-02-02 
     double *freq_by_stat;        // frequency table by station  2018-02-02 


     // scan info
     /* element size MAXOBS */
     //char (*obsstar)[11];   // Observation sourcer name
     char (*obsstar)[MAX_SRC_NAME];   // Observation sourcer name  2018-08-28
     char (*obsid)[21];     // station IDs participated in this obs
     char (*scanid)[21];    // added on 2005-01-13
     int *starid;         // Star ID  (entry at starpos table)
     int *dura;           // obs duration (sec)
     int (*start)[5];       // start time   YY, DDD, HH, MM, SS
     int nobs;                   // Number of observations (scans) (maximum is MAXOBS)
     int nobs_max;               // MAXOBS maximum nuber of scans for malloc
     int nobs_true;              // Number of observations (scans) truely included in schedule file 2009-11-19

     /* element size MAXOBS*MAXSTAT */
     //int datastart[MAXOBS][MAXSTAT];   // data start time offset 
     int **datastart;   // data start time offset 
                                       //   from obs start time (sec) 
     //int datastop[MAXOBS][MAXSTAT];    // data stop  time offset from obs start time (sec)
     int **datastop;    // data stop  time offset from obs start time (sec)
     //int station_obs[MAXOBS][MAXSTAT]; // station participated in scan info (1: obs  0: no obs)
     int **station_obs; // station participated in scan info (1: obs  0: no obs)


     char skdname[61];           // schedule file name
     char skedtype[10];          // sked type SKED|VEX|OVEX
     //double x_clock_offset;


     /* following is added  to contain early start info */ //2005-07-25
     int earlysec; // early start parameter (sec)
     /* following is added  to set sideband from operator */ //2006-03-17
     int usb; // 1: USB  0: LSB

	 /* following is added to support multi frequency table */ // 2013-01-30
	 codes_t code;

     /* pickup source name  */  // 2016-10-12
     //char pickup_source[10];   // for "apri_calc" and "skdchk"
     char pickup_source[MAX_SRC_NAME];   // for "apri_calc" and "skdchk"   // 2018-08-28
     char pickup_baseid[5];     // for "apri_calc" and "skdchk"
     
} sked_data_t;

/*
typedef struct schedule_old
{
     char expcode[11];           // experiment code
     char starnam1[MAXSTAR][11]; // star name1    MAXSTAR*11
     char starnam2[MAXSTAR][11]; // star name2
     double ra[MAXSTAR];         // star position RA  in degree
     double dec[MAXSTAR];        //       "       DEC in degree
     double epo[MAXSTAR];        //       "       Epoch
     int    nstar;               // number of stars (maximum is MAXSTAR)
     int    nstar_true;          // number of stars truely included in schedule file 2009-11-19

     char statnam[MAXSTAT][11];       // station name
     double xyz[MAXSTAT][3];          // station position (x,y,z) in m
     char statid[MAXSTAT][3];         // 1 char or 2 char station ID 
     char statid2[MAXSTAT][3];         // 1 char or 2 char station ID 
     int nstation;               // number of stations (maximum is MAXSTAT)
     int nstation_true;          // number of stations truely included in schedule file 2009-11-19

     //double freqtab[16];         // RF frequency table (Hz)
     //double pcalf[10][16];      // PCAL frequency table (Hz)  modified on 2003-07-18
     double freqtab[MAXCH];         // RF frequency table (Hz)    2016-08-10
     double pcalf[10][MAXCH];      // PCAL frequency table (Hz)   2016-08-10
                                //  up to 10 th PCAL frequency
     int nfreq;                  // number of frequency
     // scan info
     char obsstar[MAXOBS][11];   // Observation sourcer name
     int starid[MAXOBS];         // Star ID  (entry at starpos table)
     int start[MAXOBS][5];       // start time   YY, DDD, HH, MM, SS
     int dura[MAXOBS];           // obs duration (sec)
     char obsid[MAXOBS][21];     // station IDs participated in this obs
     int nobs;                   // Number of observations (scans) (maimum is MAXOBS)
     int nobs_true;              // Number of observations (scans) truely included in schedule file 2009-11-19
     char scanid[MAXOBS][21];    // added on 2005-01-13
     char skdname[61];           // schedule file name
     char skedtype[10];          // sked type SKED|VEX|OVEX
     //double x_clock_offset;

     //char sideband[16][4];       // side band  U|L or USB|LSB
     char sideband[MAXCH][4];       // side band  U|L or USB|LSB  2016-08-10
     int datastart[MAXOBS][MAXSTAT];   // data start time offset 
                                 //   from obs start time (sec) 
     int datastop[MAXOBS][MAXSTAT];    // data stop  time offset from obs start time (sec)
     int station_obs[MAXOBS][MAXSTAT]; // station participated in scan info (1: obs  0: no obs)

     // following is added  to contain early start info  //2005-07-25
     int earlysec; // early start parameter (sec)
     // following is added  to set sideband from operator  //2006-03-17
     int usb; // 1: USB  0: LSB

	 // following is added to support multi frequency table  // 2013-01-30
	 codes_t code;
     
} sked_data_old_t;
*/

typedef struct skedline  // structure for each scan data
{
    char source[10];   // radio source
    int start[5];      // start time  YY,DDD,HH,MM,SS
    int dura;          // max scan length in second
    int nstat;         // participated stations in this scan
    //char id_stat[MAXSTAT][3]; // participated station id
    char (*id_stat)[3]; // participated station id   2016-09-16
    //int dura_stat[MAXSTAT];   // scan length by station
    int *dura_stat;   // scan length by station      2016-09-16
    int nstat_max;    // 2016-09-16
} skedline_t;

/*******/
/* primitive block structure for VEX analysis*/

typedef struct global
{
    char exper[30];     // link id to $EXPER block
    char eop[30];       // link id to $EOP block
} global_t;

typedef struct exper
{
    char def[30];         // def id
    char exper_name[20];  // experiment code (name)
} exper_t;

typedef struct mode_item
{
    char mode[30];   // major link id to other block
    char id[20];     // link id to STATION block
} mode_item_t;

typedef struct fanout_def
{
    char sub_pass[2];      // sub pass character (usually A|B )
    char chan_id[20];      // link id to chan_def in $FREQ block
    char stream_info[5];   // sign | mag
    int headstack;         // head stack number
    int track[4];          // track number from first multiplex to last multiplex
    int fanout;            // fanout
} fanout_def_t;
 

typedef struct chan_def
{
    double rf;         // RF frequency (Hz)
    char sideband[2];  // net side band  U|L
    double vbw;        // band width (Hz)
    char chan_id[20];  // Channel ID link to $TRACK
    char bbc_id[20];   // BBC ID  link to $BBC
    char pcal_id[20];  // Pcal ID  link to $PHASE_CAL
} chan_def_t;

typedef struct tracks
{
    char     def[50];            // def id
    char  format[20];            // track frame format ex. Mark4
    int    multiplex;            // multiplex 1|2|4 
    int    bitssampl;            // bits/sampl
    //fanout_def_t fanout_def[32]; // fanout def info (fanout_def type)
    fanout_def_t fanout_def[64]; // fanout def info (fanout_def type) 2003-10-10
    int nfanout_def;             // number of fanout_def
} tracks_t;

typedef struct freq
{
    char   def[50];              // def id
    double samplerate;           // sampling rate (Hz/s)
    int numch;                   // number of channels
    //chan_def_t chan_def[16];     // channel def info (chan_def type)
    chan_def_t chan_def[64];     // channel def info (chan_def type)
} freq_t;

typedef struct if_def            // added on 2003-07-17
{
    char if_id[20];              // IF ID link to $BBC
    char pol[2];                 // Polarization R|L
    double lo;                   // total local frequency (Hz)
    char sideband[2];            // net side band  U|L
    double pcal_space;           // PCAL freq spacing (Hz)
    double pcal_base;            // PCAL freq base (Hz)
} if_def_t;

typedef struct if_block          // added on 2003-07-17
{
    char   def[50];              // def id
    int nif;                   // number of IF definition
    if_def_t if_def[10];         // IF def info (if_def type)
} if_block_t;

typedef struct pcal_detect
{
    char pcal_id[20];     // Pcal id link to $FREQ block
    int ntone;            // number of tones
    int tone[10];  // tone number info for pcal detection
} pcal_detect_t;

typedef struct pcal_block             // added on 2003-07-17
{
    char def[50];                   // def id
    int npcal;                         // number of PCAL definition
    pcal_detect_t pcal[20];         // PCAL def info
} pcal_block_t;

typedef struct bbc          // added on 2003-07-17
{
    char bbc_id[20];     // BBC id link to $FREQ block
    char if_id[20];     // IF id link to $IF block
    int bbcch;        // Physical BBC number
} bbc_t;

typedef struct bbc_block          // added on 2003-07-17
{
    char   def[50];       // def id
    char nbbc;     // number of BBC block
    bbc_t bbc_def[20];    // BBC def 
} bbc_block_t;

typedef struct siteclock
{
    char   def[50];        // def id
    int validity_epoch[5]; // yyyy,ddd,hh,mm,ss
    double clock_offset;   // clock offset wrt UTC  >0 for formatter tick early
    int clock_epoch[5];    // yyyy,ddd,hh,mm,ss
    double clock_rate;     // s/s
} siteclock_t;

typedef struct site
{
    char   def[50];        // def id
    char   type[20];       // site type 'fixed' or others
    char   name[20];       // site name
    char   id[3];          // 2 chars site id
    double position[3];    // site position X,Y,Z (m)
    siteclock_t sclock;    // site clock info [siteclock_t type]
} site_t;


typedef struct source
{
    char    def[50];         // def id
    char    type[20];        // star|satellite
    char    name[20];        // source name
    char    IAU_name[20];    // IAU name
    double  ra;              // RA in degree
    double  dec;             // DEC in degree
    char    epoch[10];       // epoch  J2000|B1950 etc
} source_t;

typedef struct eop
{
    char  def[50];     // def id
    int  epoch[5];     // EOP epoch yyyy, ddd, hh, mm, ss
    int    points;     // number of eop data points
    int interval_hr;   // eop data interval (unit is hour)
    double ut1[10];    // UT1-UTCs (sec)
    double xwob[10];   // Wobbling X parameters (asec)
    double ywob[10];   // Wobbling Y parameters (asec)
} eop_t;

typedef struct station_def
{
    /* Note! should be used as a member of vex_t type variable */
    char def[20];      // def id
    //char das[20];      // link id to DAS block
    //char das[MAXDAS][20];      // link id to DAS block 2009-11-19
    char (*das)[20];      // link id to DAS block 2016-09-16
    char clock[20];    // link id to CLOCK block
    char site[20];     // link id to SITE block
    char antenna[20];  // link id to ANTENNA block
    int ndas_max;      // 2016-09-16
} station_def_t;

typedef struct das_def  // added on 2005-07-25
{
    char def[50];      // def id
    int earlysec;      // tape early start param (sec)
    // other parameters are omitted
} das_def_t;

typedef struct station_in_scan
{
    char id[3];       // 2 chars station id
    int data_start;   // sec  valid data offset from start
    int data_stop;    // sec  stop time from start
} station_in_scan_t;

typedef struct scan
{
    /* Note! should be used as a member of vex_t type variable */
    char scanid[20];  // scan id (used as a Mark 5 data file name)
    char mode[30];    // link id to MODE block (describing track, freq link)
    int number;       // scan (obs) number in an experiment
    int start[5];     // scan start time   yyyy,ddd,hh,mm,ss
    char source[20];  // source name
    //station_in_scan_t station[MAXSTAT];  // scan period by station
    station_in_scan_t *station;  // scan period by station   2016-09-16
    int numstat;      // acutual stations in station[MAXSTAT}
    int nstation_max;      // MAXSTAT maximum number stations 2016-09-16
} scan_t;

typedef struct roll
{
    char    def[50];      // def ID
    char    rolldef[4];   // on|off
} roll_t;

typedef struct vexmode    // structure for MODE block data
{
    char def[30];                    // def id
    //mode_item_t mode_freq[MAXSTAT];  // link to $FREQ block info by station (link id to $STATION)
    //mode_item_t mode_track[MAXSTAT]; // link to $TRACK block info by station ( " )
    //mode_item_t mode_roll[MAXSTAT];  // link to $ROLL block info by station  ( " )
    //mode_item_t mode_pcal_detect[MAXSTAT]; // link to PCAL block info by station ( " )
    //mode_item_t mode_if[MAXSTAT]; // link to $IF block info by station ( " )
    //mode_item_t mode_bbc[MAXSTAT]; // link to $BBC block info by station ( " )
    // after 2016-09-16
    mode_item_t *mode_freq;  // link to $FREQ block info by station (link id to $STATION)
    mode_item_t *mode_track; // link to $TRACKS block info by station ( " )
    mode_item_t *mode_roll;  // link to $ROLL block info by station  ( " )
    mode_item_t *mode_pcal_detect; // link to PCAL block info by station ( " )
    mode_item_t *mode_if; // link to $IF block info by station ( " )
    mode_item_t *mode_bbc; // link to $BBC block info by station ( " )
    int nmode;                // used only for count mode_freq and mode_track count
} vexmode_t;
   
/*
typedef struct vex_old
{
    char vex_file_name[60];    // VEX file name
    char block[50][20];        // found $block name table  50x20chars
    int nblock;                // found $block number
    char flag[10];             // vex type identifier VEX|OVEX
    global_t global;           // $GLOBAL block info
    exper_t exper[10];         // $EXPER block info up to 10
    int nexper;                // number of def in $EXPER block
    vexmode_t mode[10];        // $MODE block info
    int nmode;                 // number of def in $MODE block
    site_t site[MAXSTAT];      // $SITE block info
    int  numstat;              // number of def in $SITE block (maximum is MAXSTAT)
    int  numstat_true;         // number of def truely in $SITE block 2009-11-19
    source_t source[MAXSTAR];  // $SOURCE block info
    int  numstar;              // number of def in $SOURCE block (maximum is MAXSTAR)
    int  numstar_true;         // number of def truely in $SOURCE block 2009-11-19
    //freq_t freq_def[5];        // $FREQ block info
    freq_t freq_def[20];        // $FREQ block info   2003-07-04
    int  nfreq_def;            // number of def in $FREQ block
    tracks_t track_def[10];    // $TRACK block info
    int ntrack_def;            // number of def in $TRACK block
    scan_t  scan[MAXOBS];      // $SCHED info
    int  numobs;               // number of observations in $SCHED block (maximum is MAXOBS)
    int  numobs_true;               // number of observations truely in $SCHED block 2009-11-19
    station_def_t station[MAXSTAT]; // $STATION block info
    int nstation;                   // number of def in $STATION block (maximum is MAXSTAT)
    int nstation_true;              // number of def truely in $STATION block 2009-11-19
    siteclock_t siteclock[MAXSTAT]; // $CLOCK block info
    int nclock;                     // number of def in $CLOCK block
    eop_t   eop[10];           // $EOP block info
    int neop;                  // number of def in $EOP block
    roll_t  roll[10];          // $ROLL block info
    int nroll;                 // number of def in $ROLL block
//    pcal_block_t pcal[20];    // $PCAL block info
    pcal_block_t pcal[MAXPCAL_B];    // $PCAL block info   2016-09-14
    int npcal;                 // number of def in $PCAL block
//    bbc_block_t  bbc[20];    // $BBC block info
    bbc_block_t  bbc[MAXBBC_B];    // $BBC block info     2016-09-14
    int nbbc;                 // number of def in $BBC block
//    if_block_t  ifb[20];    // $IF block info
    if_block_t  ifb[MAXIF_B];    // $IF block info        2016-09-14
    int nifb;                 // number of def in $IF block

    // following is added  to contain early start info//2005-07-25

    //das_def_t das[20]; // DAS info including early start parameter
    //das_def_t das[40]; // DAS info including early start parameter //2005-05-09
    das_def_t das[MAXDAS]; // DAS info including early start parameter //2009-11-13
    int ndas;          // number of def in $DAS block
    int ndas_true;     // number of def truely in $DAS block 2009-11-19
} vex_old_t;
*/

typedef struct vex
{
    char vex_file_name[60];    // VEX file name
    //char block[50][20];        // found $block name table  50x20chars
    char (*block)[20];        // found $block name table  50x20chars
    int nblock_max;            // maximum number of $block number  2016-09-16
    int nblock;                // found $block number
    char flag[10];             // vex type identifier VEX|OVEX
    global_t global;           // $GLOBAL block info
    //exper_t exper[10];         // $EXPER block info up to 10
    exper_t *exper;            // $EXPER block info up to 10
    int nexper_max;            // maximum number of def in $EXPER block 2016-09-16
    int nexper;                // number of def in $EXPER block
    //vexmode_t mode[10];        // $MODE block info
    vexmode_t *mode;           // $MODE block info
    int nmode_max;             // maximum number of def in $MODE block  2016-09-16
    int nmode;                 // number of def in $MODE block
    //site_t site[MAXSTAT];      // $SITE block info
    site_t *site;              // $SITE block info
    int  numstat;              // number of def in $SITE block (maximum is MAXSTAT)
    int  numstat_max;          // MAXSTAT maximum number of def in $SITE block (maximum is MAXSTAT) 2016-09-15
    int  numstat_true;         // number of def truely in $SITE block 2009-11-19
    //source_t source[MAXSTAR];  // $SOURCE block info
    source_t *source;  // $SOURCE block info
    int  numstar;              // number of def in $SOURCE block (maximum is MAXSTAR)
    int  numstar_max;          // MAXSTAR maximum number of def in $SOURCE block (maximum is MAXSTAR) 2016-09-16
    int  numstar_true;         // number of def truely in $SOURCE block 2009-11-19
    //freq_t freq_def[20];        // $FREQ block info   2003-07-04
    freq_t *freq_def;        // $FREQ block info   2016-09-16
    int  nfreq_def_max;            // maximum number of def in $FREQ block  2016-09-16
    int  nfreq_def;            // number of def in $FREQ block
    //tracks_t track_def[10];    // $TRACK block info
    tracks_t *track_def;    // $TRACK block info
    int ntrack_def_max;        // maximum number of def in $TRACK block  2016-09-16
    int ntrack_def;            // number of def in $TRACK block
    //scan_t  scan[MAXOBS];      // $SCHED info
    scan_t  *scan;      // $SCHED info
    int  numobs;               // number of observations in $SCHED block (maximum is MAXOBS)
    int  numobs_max;               // MAXOBS maximum number of observations in $SCHED block (maximum is MAXOBS)
    int  numobs_true;               // number of observations truely in $SCHED block 2009-11-19
    //station_def_t station[MAXSTAT]; // $STATION block info
    station_def_t *station; // $STATION block info
    int nstation;                   // number of def in $STATION block (maximum is MAXSTAT)
    int nstation_max;               // MAXSTAT maximum number of def in $STATION block (maximum is MAXSTAT)
    int nstation_true;              // number of def truely in $STATION block 2009-11-19
    //siteclock_t siteclock[MAXSTAT]; // $CLOCK block info
    siteclock_t *siteclock; // $CLOCK block info
    int nclock_max;                // =MAXSTAT maximum number of def in $CLOCK block
    int nclock;                     // number of def in $CLOCK block
    //eop_t   eop[10];           // $EOP block info
    eop_t   *eop;           // $EOP block info
    int neop_max;              // maximum number of def in $EOP block  2016-09-16
    int neop;                  // number of def in $EOP block
    //roll_t  roll[10];          // $ROLL block info
    roll_t  *roll;          // $ROLL block info
    int nroll_max;                 // maximum number of def in $ROLL block
    int nroll;                 // number of def in $ROLL block
//    pcal_block_t pcal[20];    // $PCAL block info
//    pcal_block_t pcal[MAXPCAL_B];    // $PCAL block info   2016-09-14
    pcal_block_t *pcal;    // $PCAL block info   2016-09-14
    int npcal_max;             // MAXPCAL_B maximum number of def in $PCAL block 2016-09-16
    int npcal;                 // number of def in $PCAL block
//    bbc_block_t  bbc[20];    // $BBC block info
//    bbc_block_t  bbc[MAXBBC_B];    // $BBC block info     2016-09-14
    bbc_block_t  *bbc;    // $BBC block info     2016-09-14
    int nbbc_max;             // MAXBBC_B maximum number of def in $BBC block  2019-09-16
    int nbbc;                 // number of def in $BBC block
//    if_block_t  ifb[20];    // $IF block info
    //if_block_t  ifb[MAXIF_B];    // $IF block info        2016-09-14
    if_block_t  *ifb;    // $IF block info        2016-09-14
    int nifb_max;               // MAXIF_B maximum number of def in $IF block
    int nifb;                 // number of def in $IF block

    /* following is added  to contain early start info */ //2005-07-25

    //das_def_t das[20]; // DAS info including early start parameter
    //das_def_t das[40]; // DAS info including early start parameter //2005-05-09
    //das_def_t das[MAXDAS]; // DAS info including early start parameter //2009-11-13
    das_def_t *das; // DAS info including early start parameter //2009-11-13
    int ndas_max;      // MAXDAS maximum number of def in $DAS block  2016-09-16
    int ndas;          // number of def in $DAS block
    int ndas_true;     // number of def truely in $DAS block 2009-11-19
} vex_t;



typedef struct sidtable
{
    char sid[10];
    char site[20]; // site name 2005-01-13
} sidtable_t;

typedef struct freq_info
{
    char chanid[10];       // channel ID
    char sideband[10];    // net sideband  usualy U|L or USB|LSB
    double rf;            // RF frequency (Hz)
    double pcalf[10];     // PCAL frequency in video (up to 10th tone) 2003-07-16
    int npcal;            //       valid number of data in pcalf
    double vbw;           // Video frequency band width (Hz)
    int nch;              // channel # for this info
    int chmax;            // maximum channels in original table
    int bbc;              // BBC# 2004-12-25
    char bbcid[20];       //  BBC ID  2004-12-25
} freq_info_t;


typedef struct track_freq_sub
{
    int fan_out_pos;      // 1 for 1st   2 for secondary
    int headstack;    // head stack position  1,2,3,4  added 2004-09-13
    int track;            // track number
    char chan_id[20];     // chan id
    char sideband[10];    // net sideband  usualy U|L or USB|LSB
    char bitstream[10];       //  sign|mag
    double rf;            // RF frequency (Hz)
    double pcalf[10];     // PCAL frequency in video (up to 10th tone) 2003-07-16
    int npcal;            //       valid number of data in pcalf
    double vbw;           // Video frequency band width (Hz)
    int bbc;              // BBC#  2004-12-25
    char bbcid[20];       //  BBC ID  2004-12-25
    char sub_pass[2];    // pass info (A|B) 2004-06-29
} track_freq_sub_t;

typedef struct track_freq
{
    //track_freq_sub_t trk[32];
    track_freq_sub_t trk[64];     // 2003-10-10
    char trackdef[30];
    char freqdef[30];
    int ntrack;
    int nfreq;
    int adbit;
    double samplrate;     // Sampling rate sampls/sec
    int fanout;        // fanout 1|2|4
    char roll[30];
} track_freq_t;

typedef struct vparam
{
    int  n;          //number of param
    char param[60][128];   // 60x128 characters
} vparam_t;



#ifdef __cplusplus
extern "C"{
#endif 

double clock_get(siteclock_t clock, int *prt);  //2003-07-13
void sa_clock_get(vex_t *vex, char *stat_id1, char *stat_id2, int *prt, 
                  double *coffset, double *crate, double *xcoff);  //2005-05-19
void eop_get2(eop_t eop, int *prt, double *ut1, double *xwob, double *ywob);  //2003-07-13
void eop_get(vex_t *vex, int *prt, 
                  double *ut1, double *xwob, double *ywob);  //2003-07-13
void parsetime1(char *ctime, int *itime);  //2003-03-21
void asc_ra2deg(char *ra, double *radeg); //2003-03-21
void asc_dec2deg(char *dec, double *decdeg); //2003-03-21
int vex_parse(char *cbuf, vparam_t *vparam);  //2003-03-21
int GLOBAL_ana(FILE *fin, char *line, vex_t *vex); //2003-03-21
int EXPER_ana(FILE *fin, char *line, vex_t *vex);  //2003-03-21
int SKED_ana(FILE* fin, char *line, vex_t *vex); //2003-03-21
int sub_sked_ana(FILE* fin, char *line, scan_t *scan);  //2009-11-19
int TRACKS_ana(FILE* fin, char *line, vex_t *vex); //2003-03-21
int SITE_ana(FILE* fin, char *line, vex_t *vex); //2003-03-21
int sub_site_ana(FILE* fin, char *line, site_t *site); //2009-11-19
int STATION_ana(FILE* fin, char *line, vex_t *vex); //2003-03-21
int sub_station_ana(FILE* fin, char *line, station_def_t *station);  //2009-11-19
int SOURCE_ana(FILE* fin, char *line, vex_t *vex); //2003-03-21
int sub_source_ana(FILE* fin, char *line, source_t *source);  //2009-11-19
int CLOCK_ana(FILE* fin, char *line, vex_t *vex); //2003-03-21
int EOP_ana(FILE* fin, char *line, vex_t *vex); //2003-03-21
int ROLL_ana(FILE* fin, char *line, vex_t *vex); //2003-03-21
int FREQ_ana(FILE* fin, char *line, vex_t *vex); //2003-03-21
int MODE_ana(FILE* fin, char *line, vex_t *vex);  //2003-07-17
int PCAL_ana(FILE* fin, char *line, vex_t *vex); /* 2004-06-17 */
int BBC_ana(FILE* fin, char *line, vex_t *vex); //2003-07-17
int IF_ana(FILE* fin, char *line, vex_t *vex); //2003-07-17
int DAS_ana(FILE* fin, char *line, vex_t *vex); //2005-07-25
int sub_das_ana(FILE* fin, char *line, das_def_t *das);  //2009-11-19
void MODE_mon(vex_t *vex);  //2003-03-21
void SKED_mon(vex_t *vex);  //2003-03-21
void TRACKS_mon(vex_t *vex);  //2003-03-21
void SITE_mon(vex_t *vex);  //2003-03-21
void STATION_mon(vex_t *vex);  //2003-03-21
void SOURCE_mon(vex_t *vex);  //2003-03-21
void FREQ_mon(vex_t *vex);  //2003-03-21
void CLOCK_mon(vex_t *vex);  //2003-03-21
void CLOCK_mon2(vex_t *vex, char *stat_id);  //2003-07-13
//void CLOCK_get(vex_t *vex, char *stat_id, siteclock_t *clock);  //2003-07-19
int CLOCK_get(vex_t *vex, char *stat_id, siteclock_t *clock);  //2018-02-02
void EOP_mon(vex_t *vex);  //2003-03-21
void EOP_mon2(vex_t *vex);  //2003-03-21
void PCAL_mon(vex_t *vex);  //2003-07-16
void BBC_mon(vex_t *vex);  //2003-07-16
void IF_mon(vex_t *vex);  //2003-07-16
void DAS_mon(vex_t *vex);  //2005-07-25
void vex_read(vex_t *vex); 
int search_site(vex_t *vex, char *key, site_t *site);  /* 2004-06-17 */
int search_site2(vex_t *vex, char *key, site_t *site);  /* 2005-01-12 */
freq_info_t vex_freq_get(vex_t *vex, char *mode, char *sid, char *chanid);
freq_info_t vex_chinfo_get(vex_t *vex, char *mode, char *sid, int nch);
int search_track_freq(vex_t *vex, char *mode, char *sid, track_freq_t *trfreq);
int search_track_freq2(vex_t *vex, char *mode, char *sid, track_freq_t *trfreq);
void create_siteid_table(vex_t *vex, int *nstation, sidtable_t *siteid);  //2003-03-21
int check_block(vex_t *vex);  //2003-03-21


/*******/
int line_parse(char *line, vparam_t *vparam);  //2005-07-25

void SkdMonit(sked_data_t *skd);
void stcode_get(char *stcode_file, char *sid2, char *sid1);
void vex2sked(vex_t *vex, char *stcode_file, sked_data_t *skd);
void vex2sked2(vex_t *vex, sked_data_t *skd);
void vex2sked3(vex_t *vex, char *stat_id, sked_data_t *skd);
void skedlineana(char *line, skedline_t *skl);
int readskd_source(FILE *fin, char *line, sked_data_t *skd);
int readskd_station(FILE *fin, char *line, sked_data_t *skd);
//void parseSBinfo(char *s, int *tus, int *tls);
void parseSBinfo(char *s, int *tus, int *tls, int *tum, int *tlm);  // 2016-08-15
int readskd_freq(FILE *fin, char *line, sked_data_t *skd);
int readskd_sked(FILE *fin, char *line, sked_data_t *skd);
int readskd_param(FILE *fin, char *line, sked_data_t *skd); //added on 2005-07-25
void readskd(sked_data_t *skd);   
void readskd_for_nozomi(char *skdname, char *baseid, char *apefile, int *iok);
void check_sked_type(char* skdname, char* skedtype);
void pickup_skd(sked_data_t *skd, char *stat_id);
void setscanid_skd(sked_data_t *skd);
void scan_file_naming(obs_file_naming_t *ofn, char *fname);

int TrimTK(char *s); //2013-01-28 
void pick_up_sked_codes_section(char *skdname, codes_t *code);  //2013-01-28   
int readskd_codes(FILE *fin, char *line, codes_t *code);  // 2013-01-28

// added on 2016-09-14 and after
void  get_nelement_VEX(vex_t *vex);
int get_VEX_nKEY(FILE* fin, char *keyword);
void  get_nelement_SKED(sked_data_t *skd);
int get_SKED_nKEY(FILE* fin, char *keyword);

void SKED_malloc(sked_data_t *skd);
void SKED_free(sked_data_t *skd);
void VEX_malloc(vex_t *vex);
void VEX_free(vex_t *vex);

void setup_ch_alloc_table_by_station(vex_t *vex, char *mode, sked_data_t *skd); // 2017-09-03
int setup_freq_table_for_sid_vex(vex_t *vex, sked_data_t *skd, char *stat_id); // 2018-02-01 (not completed)

#ifdef __cplusplus
}
#endif

#endif
